from insights.parsers import df, ParseException
from insights.tests import context_wrap

import pytest

DF_LI = """
Filesystem        Inodes  IUsed     IFree IUse% Mounted on
/dev/mapper/vg_lxcrhel6sat56-lv_root

                 6275072 124955   6150117    2% /

devtmpfs         1497120    532   1496588    1% /dev
tmpfs            1499684    331   1499353    1% /dev/shm
tmpfs            1499684    728   1498956    1% /run
tmpfs            1499684     16   1499668    1% /sys/fs/cgroup
tmpfs            1499684     54   1499630    1% /tmp
/dev/sda2      106954752 298662 106656090    1% /home
/dev/sda1         128016    429    127587    1% /boot
tmpfs            1499684      6   1499678    1% /V M T o o l s
tmpfs            1499684     15   1499669    1% /VM Tools
""".strip()


def test_df_li():
    df_list = df.DiskFree_LI(context_wrap(DF_LI))
    assert len(df_list) == 10
    assert len(df_list.mounts) == 10
    assert len(df_list.filesystems) == 5
    assert '/home' in df_list.mounts
    r = df.Record(
        filesystem='/dev/sda2',
        total='106954752',
        used='298662',
        available='106656090',
        capacity='1%',
        mounted_on='/home'
    )

    assert df_list.get_mount('/home') == r
    assert '/dev/sda2' in df_list.filesystems
    assert len(df_list.get_filesystem('/dev/sda2')) == 1
    assert df_list.get_filesystem('/dev/sda2')[0] == r
    assert len(df_list.get_filesystem('tmpfs')) == 6
    assert df_list.get_mount('/dev').filesystem == 'devtmpfs'
    assert df_list.get_mount('/run').total == '1499684'
    assert df_list.get_mount('/tmp').used == '54'
    assert df_list.get_mount('/boot').available == '127587'
    assert df_list.get_filesystem('/dev/sda2')[0].capacity == '1%'
    assert df_list.get_filesystem('/dev/sda2')[0].available == '106656090'
    assert df_list.get_filesystem('devtmpfs')[0].mounted_on == '/dev'
    assert df_list.get_mount('/V M T o o l s').available == '1499678'
    assert df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')[0].mounted_on == '/'
    sorted_mount_names = sorted([
        '/', '/dev', '/dev/shm', '/run', '/sys/fs/cgroup', '/tmp', '/home',
        '/boot', '/V M T o o l s', '/VM Tools'
    ])
    assert sorted([d.mounted_on for d in df_list]) == sorted_mount_names
    assert sorted(df_list.mount_names) == sorted_mount_names
    assert sorted(df_list.filesystem_names) == sorted([
        '/dev/mapper/vg_lxcrhel6sat56-lv_root', 'devtmpfs', 'tmpfs',
        '/dev/sda2', '/dev/sda1'
    ])

    # Test get_path
    # Root mount point works:
    assert df_list.get_dir('/') == df_list.get_mount('/')
    # Mount point underneath root works:
    assert df_list.get_dir('/dev') == df_list.get_mount('/dev')
    # Directory underneath sub-mount:
    assert df_list.get_dir('/boot/grub2') == df_list.get_mount('/boot')
    # Directory with / suffix:
    assert df_list.get_dir('/boot/grub2/') == df_list.get_mount('/boot')
    # File path also works:
    assert df_list.get_dir('/boot/grub2/grub.cfg') == df_list.get_mount('/boot')
    # Relative path returns None
    assert df_list.get_dir('var/lib') is None
    # Invalid path returns None
    assert df_list.get_dir('"') is None


DF_ALP = """
/bin/df: '/vobs/GEMS': No such file or directory
/bin/df: '/vobs/NT/TFax': No such file or directory
Filesystem                           1024-blocks      Used Available Capacity Mounted on
/dev/mapper/vg_lxcrhel6sat56-lv_root    98571884   4244032  89313940       5% /
sysfs                                          0         0         0        - /sys
proc                                           0         0         0        - /proc
devtmpfs                                 5988480         0   5988480       0% /dev
securityfs                                     0         0         0        - /sys/kernel/security
tmpfs                                    5998736    491660   5507076       9% /dev/shm
devpts                                         0         0         0        - /dev/pts
tmpfs                                    5998736      1380   5997356       1% /run

tmpfs                                    5998736         0   5998736       0% /sys/fs/cgroup
""".strip()


def test_df_alP():
    df_list = df.DiskFree_ALP(context_wrap(DF_ALP))
    assert len(df_list) == 9
    assert len(df_list.mounts) == 9
    assert len(df_list.filesystems) == 7
    assert '/' in df_list.mounts
    r = df.Record(
        filesystem='/dev/mapper/vg_lxcrhel6sat56-lv_root',
        total='98571884',
        used='4244032',
        available='89313940',
        capacity='5%',
        mounted_on='/'
    )
    assert df_list.get_mount('/') == r
    assert '/dev/mapper/vg_lxcrhel6sat56-lv_root' in df_list.filesystems
    assert len(df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')) == 1
    assert df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')[0] == r
    assert len(df_list.get_filesystem('tmpfs')) == 3
    assert df_list.get_mount('/sys').filesystem == 'sysfs'
    assert df_list.get_mount('/proc').total == '0'
    assert df_list.get_mount('/dev').used == '0'
    assert df_list.get_mount('/run').available == '5997356'
    assert df_list.get_mount('/sys/fs/cgroup').capacity == '0%'
    assert df_list.get_mount('/').filesystem == '/dev/mapper/vg_lxcrhel6sat56-lv_root'
    assert df_list.get_mount('/').capacity == '5%'

    # Test get_path
    # Root mount point works:
    assert df_list.get_dir('/') == df_list.get_mount('/')
    # Mount point underneath root works:
    assert df_list.get_dir('/dev') == df_list.get_mount('/dev')
    # Directory underneath sub-mount:
    assert df_list.get_dir('/dev/v4l') == df_list.get_mount('/dev')
    # Directory with / suffix:
    assert df_list.get_dir('/dev/v4l/') == df_list.get_mount('/dev')
    # File path also works:
    assert df_list.get_dir('/dev/v4l/adapter0/control0.cfg') == df_list.get_mount('/dev')
    # Relative path returns None
    assert df_list.get_dir('dev/sys') is None
    # Invalid path returns None
    assert df_list.get_dir('"') is None


DF_AL = """
Filesystem                             1K-blocks      Used Available     Use% Mounted on
/dev/mapper/vg_lxcrhel6sat56-lv_root    98571884   4244032  89313940       5% /
sysfs                                          0         0         0        - /sys
proc                                           0         0         0        - /proc
devtmpfs                                 5988480         0   5988480       0% /dev
securityfs                                     0         0         0        - /sys/kernel/security

tmpfs                                    5998736    491660   5507076       9% /dev/shm
devpts                                         0         0         0        - /dev/pts
tmpfs                                    5998736      1380   5997356       1% /run
tmpfs                                    5998736         0   5998736       0% /sys/fs/cgroup
""".strip()


def test_df_al():
    df_list = df.DiskFree_AL(context_wrap(DF_AL))
    assert len(df_list) == 9
    assert len(df_list.mounts) == 9
    assert len(df_list.filesystems) == 7
    assert '/' in df_list.mounts
    r = df.Record(
        filesystem='/dev/mapper/vg_lxcrhel6sat56-lv_root',
        total='98571884',
        used='4244032',
        available='89313940',
        capacity='5%',
        mounted_on='/'
    )
    assert df_list.get_mount('/') == r
    assert '/dev/mapper/vg_lxcrhel6sat56-lv_root' in df_list.filesystems
    assert len(df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')) == 1
    assert df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')[0] == r
    assert len(df_list.get_filesystem('tmpfs')) == 3
    assert df_list.get_mount('/sys').filesystem == 'sysfs'
    assert df_list.get_mount('/proc').total == '0'
    assert df_list.get_mount('/dev').used == '0'
    assert df_list.get_mount('/run').available == '5997356'
    assert df_list.get_mount('/sys/fs/cgroup').capacity == '0%'
    assert df_list.get_mount('/').filesystem == '/dev/mapper/vg_lxcrhel6sat56-lv_root'
    assert df_list.get_mount('/').capacity == '5%'

    # Test get_path
    # Root mount point works:
    assert df_list.get_dir('/') == df_list.get_mount('/')
    # Mount point underneath root works:
    assert df_list.get_dir('/dev') == df_list.get_mount('/dev')
    # Directory underneath sub-mount:
    assert df_list.get_dir('/dev/v4l') == df_list.get_mount('/dev')
    # Directory with / suffix:
    assert df_list.get_dir('/dev/v4l/') == df_list.get_mount('/dev')
    # File path also works:
    assert df_list.get_dir('/dev/v4l/adapter0/control0.cfg') == df_list.get_mount('/dev')
    # Relative path returns None
    assert df_list.get_dir('dev/sys') is None
    # Invalid path returns None
    assert df_list.get_dir('"') is None


DF_AL_BAD = """
Filesystem                             1K-blocks      Used Available     Use% Mounted on
/dev/mapper/vg_lxcrhel6sat56-lv_root    98571884   4244032  89313940       5% /
sysfs                                          0
"""


def test_df_al_bad():
    with pytest.raises(ParseException) as exc:
        df_list = df.DiskFree_AL(context_wrap(DF_AL_BAD))
        assert len(df_list) == 2
    assert 'Could not parse line' in str(exc)



"""
Collect all the interesting data for analysis
"""
from __future__ import absolute_import
import os
import errno
import json
from . import archive
import logging
import copy
import glob
import six
import shlex
from subprocess import Popen, PIPE, STDOUT
from tempfile import NamedTemporaryFile

from insights.util import mangle
from ..contrib.soscleaner import SOSCleaner
from .utilities import _expand_paths
from .constants import InsightsConstants as constants
from .insights_spec import InsightsFile, InsightsCommand

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)
# python 2.7
SOSCLEANER_LOGGER = logging.getLogger('soscleaner')
SOSCLEANER_LOGGER.setLevel(logging.ERROR)
# python 2.6
SOSCLEANER_LOGGER = logging.getLogger('insights-client.soscleaner')
SOSCLEANER_LOGGER.setLevel(logging.ERROR)


class DataCollector(object):
    '''
    Run commands and collect files
    '''

    def __init__(self, config, archive_=None, mountpoint=None):
        self.config = config
        self.archive = archive_ if archive_ else archive.InsightsArchive()
        self.mountpoint = '/'
        if mountpoint:
            self.mountpoint = mountpoint
        self.hostname_path = None

    def _write_branch_info(self, branch_info):
        logger.debug("Writing branch information to archive...")
        self.archive.add_metadata_to_archive(
            json.dumps(branch_info), '/branch_info')

    def _write_display_name(self):
        if self.config.display_name:
            logger.debug("Writing display_name to archive...")
            self.archive.add_metadata_to_archive(
                self.config.display_name, '/display_name')

    def _run_pre_command(self, pre_cmd):
        '''
        Run a pre command to get external args for a command
        '''
        logger.debug('Executing pre-command: %s', pre_cmd)
        try:
            pre_proc = Popen(pre_cmd, stdout=PIPE, stderr=STDOUT, shell=True)
        except OSError as err:
            if err.errno == errno.ENOENT:
                logger.debug('Command %s not found', pre_cmd)
            return
        stdout, stderr = pre_proc.communicate()
        the_return_code = pre_proc.poll()
        logger.debug("Pre-command results:")
        logger.debug("STDOUT: %s", stdout)
        logger.debug("STDERR: %s", stderr)
        logger.debug("Return Code: %s", the_return_code)
        if the_return_code != 0:
            return []
        if six.PY3:
            stdout = stdout.decode('utf-8')
        return stdout.splitlines()

    def _parse_file_spec(self, spec):
        '''
        Separate wildcard specs into more specs
        '''
        # separate wildcard specs into more specs
        if '*' in spec['file']:
            expanded_paths = _expand_paths(spec['file'])
            if not expanded_paths:
                return []
            expanded_specs = []
            for p in expanded_paths:
                _spec = copy.copy(spec)
                _spec['file'] = p
                expanded_specs.append(_spec)
            return expanded_specs

        else:
            return [spec]

    def _parse_glob_spec(self, spec):
        '''
        Grab globs of things
        '''
        some_globs = glob.glob(spec['glob'])
        if not some_globs:
            return []
        el_globs = []
        for g in some_globs:
            _spec = copy.copy(spec)
            _spec['file'] = g
            el_globs.append(_spec)
        return el_globs

    def _blacklist_check(self, cmd):
        def _get_nested_parts(cmd):
            parts = shlex.split(cmd.replace(';', ' '))
            all_parts = parts[:]
            for p in parts:
                if len(shlex.split(p)) > 1:
                    all_parts += _get_nested_parts(p)
            return all_parts

        cmd_parts = _get_nested_parts(cmd)
        return len(set.intersection(set(cmd_parts),
                   constants.command_blacklist)) > 0

    def _parse_command_spec(self, spec, precmds):
        '''
        Run pre_commands
        '''
        if 'pre_command' in spec:
            precmd_alias = spec['pre_command']
            try:
                precmd = precmds[precmd_alias]

                if self._blacklist_check(precmd):
                    raise RuntimeError("Command Blacklist: " + precmd)

                args = self._run_pre_command(precmd)
                logger.debug('Pre-command results: %s', args)

                expanded_specs = []
                for arg in args:
                    _spec = copy.copy(spec)
                    _spec['command'] = _spec['command'] + ' ' + arg
                    expanded_specs.append(_spec)
                return expanded_specs
            except LookupError:
                logger.debug('Pre-command %s not found. Skipping %s...',
                             precmd_alias, spec['command'])
                return []
        else:
            return [spec]

    def run_collection(self, conf, rm_conf, branch_info):
        '''
        Run specs and collect all the data
        '''
        if rm_conf is None:
            rm_conf = {}
        logger.debug('Beginning to run collection spec...')
        exclude = None
        if rm_conf:
            try:
                exclude = rm_conf['patterns']
                logger.warn("WARNING: Skipping patterns found in remove.conf")
            except LookupError:
                logger.debug('Patterns section of remove.conf is empty.')

        for c in conf['commands']:
            # remember hostname archive path
            if c.get('symbolic_name') == 'hostname':
                self.hostname_path = os.path.join(
                    'insights_commands', mangle.mangle_command(c['command']))
            rm_commands = rm_conf.get('commands', [])
            if c['command'] in rm_commands or c.get('symbolic_name') in rm_commands:
                logger.warn("WARNING: Skipping command %s", c['command'])
            elif self.mountpoint == "/" or c.get("image"):
                cmd_specs = self._parse_command_spec(c, conf['pre_commands'])
                for s in cmd_specs:
                    cmd_spec = InsightsCommand(self.config, s, exclude, self.mountpoint)
                    self.archive.add_to_archive(cmd_spec)
        for f in conf['files']:
            rm_files = rm_conf.get('files', [])
            if f['file'] in rm_files or f.get('symbolic_name') in rm_files:
                logger.warn("WARNING: Skipping file %s", f['file'])
            else:
                file_specs = self._parse_file_spec(f)
                for s in file_specs:
                    # filter files post-wildcard parsing
                    if s['file'] in rm_conf.get('files', []):
                        logger.warn("WARNING: Skipping file %s", s['file'])
                    else:
                        file_spec = InsightsFile(s, exclude, self.mountpoint)
                        self.archive.add_to_archive(file_spec)
        if 'globs' in conf:
            for g in conf['globs']:
                glob_specs = self._parse_glob_spec(g)
                for g in glob_specs:
                    if g['file'] in rm_conf.get('files', []):
                        logger.warn("WARNING: Skipping file %s", g)
                    else:
                        glob_spec = InsightsFile(g, exclude, self.mountpoint)
                        self.archive.add_to_archive(glob_spec)
        logger.debug('Spec collection finished.')

        # collect metadata
        logger.debug('Collecting metadata...')
        self._write_branch_info(branch_info)
        self._write_display_name()
        logger.debug('Metadata collection finished.')

    def done(self, conf, rm_conf):
        """
        Do finalization stuff
        """
        if self.config.obfuscate:
            cleaner = SOSCleaner(quiet=True)
            clean_opts = CleanOptions(
                self.config, self.archive.tmp_dir, rm_conf, self.hostname_path)
            fresh = cleaner.clean_report(clean_opts, self.archive.archive_dir)
            if clean_opts.keyword_file is not None:
                os.remove(clean_opts.keyword_file.name)
                logger.warn("WARNING: Skipping keywords found in remove.conf")
            return fresh[0]
        return self.archive.create_tar_file()


class CleanOptions(object):
    """
    Options for soscleaner
    """
    def __init__(self, config, tmp_dir, rm_conf, hostname_path):
        self.report_dir = tmp_dir
        self.domains = []
        self.files = []
        self.quiet = True
        self.keyword_file = None
        self.keywords = None

        if rm_conf:
            try:
                keywords = rm_conf['keywords']
                self.keyword_file = NamedTemporaryFile(delete=False)
                self.keyword_file.write("\n".join(keywords))
                self.keyword_file.flush()
                self.keyword_file.close()
                self.keywords = [self.keyword_file.name]
                logger.debug("Attmpting keyword obfuscation")
            except LookupError:
                pass

        if config.obfuscate_hostname:
            # default to its original location
            self.hostname_path = hostname_path or 'insights_commands/hostname'
        else:
            self.hostname_path = None



from insights.parsers.virt_what import VirtWhat
from insights.tests import context_wrap

errors = ["virt-what: virt-what-cpuid-helper program not found in $PATH"]


VW_OUT1 = """
kvm
""".strip()

# baremetal retuns blank
VW_OUT2 = """

""".strip()

# occasionally we have more than 1 line of output
VW_OUT3 = """
xen
xen-dom0
aws
""".strip()


def test_kvm():
    v1 = VirtWhat(context_wrap(VW_OUT1))
    assert v1.is_virtual is True
    assert v1.generic == "kvm"
    assert v1.specifics == []
    assert v1.errors == []


def test_bEaR_metal():
    v2 = VirtWhat(context_wrap(VW_OUT2))
    assert v2.is_physical is True
    assert v2.generic == "baremetal"
    assert v2.specifics == []
    assert v2.errors == []


def test_xen():
    v3 = VirtWhat(context_wrap(VW_OUT3))
    assert v3.is_virtual is True
    assert v3.generic == "xen"
    assert "xen-dom0" in v3
    assert "xen" in v3
    assert "aws" in v3
    assert v3.errors == []


def test_error_handling():
    v = VirtWhat(context_wrap(errors[0]))
    assert v.generic == ''
    assert v.specifics == []
    assert v.is_virtual is None
    assert v.is_physical is None
    assert v.errors == errors



"""
Swift Conf Files - file ``/etc/swift/``
=======================================

This module provides parsers for swift config files under /etc/swift directory.

SwiftObjectExpirerConf - file ``/etc/swift/object-expirer.conf``
----------------------------------------------------------------

SwiftProxyServerConf - file ``/etc/swift/proxy-server.conf``
------------------------------------------------------------

SwiftConf - file ``/etc/swift/swift.conf``
------------------------------------------------------------
"""

from .. import IniConfigFile, parser
from ..specs import Specs


@parser(Specs.swift_proxy_server_conf)
class SwiftProxyServerConf(IniConfigFile):
    """
    This class is to parse the content of the ``/etc/swift/proxy-server.conf``.

    The swift proxy - server configuration file
    ``/etc/swift/proxy-server.conf`` is in the standard 'ini' format and is
    read by the :py:class:`insights.core.IniConfigFile` parser class.

    Sample configuration file::

        [DEFAULT]
        bind_port = 8080
        bind_ip = 172.20.15.20
        workers = 0
        [pipeline:main]
        pipeline = catch_errors healthcheck proxy-logging cache ratelimit

        [app:proxy-server]
        use = egg:swift  # proxy
        set log_name = proxy-server
        set log_facility = LOG_LOCAL1

        [filter:catch_errors]
        use = egg:swift  # catch_errors

    Examples:
        >>> 'app:proxy-server' in proxy_server_conf
        True
        >>> proxy_server_conf.get('filter:catch_errors', 'use') == 'egg:swift#catch_errors'
        True
        >>> proxy_server_conf.getint('DEFAULT', 'bind_port')
        8080
    """
    pass


@parser(Specs.swift_object_expirer_conf)
class SwiftObjectExpirerConf(IniConfigFile):
    """
    This class is to parse the content of the ``/etc/swift/object-expirer.conf``.

    `/etc/swift/object-expirer.conf`` is in the standard 'ini' format and is
    read by the :py:class:`insights.core.IniConfigFile` parser class.

    Sample configuration file::

        [DEFAULT]

        [object-expirer]
        # auto_create_account_prefix = .
        auto_create_account_prefix = .
        process=0
        concurrency=1
        recon_cache_path=/var/cache/swift
        interval=300
        reclaim_age=604800
        report_interval=300
        processes=0
        expiring_objects_account_name=expiring_objects

        [pipeline:main]
        pipeline = catch_errors cache proxy-server

        [app:proxy-server]
        use = egg:swift#proxy

        [filter:cache]
        use = egg:swift#memcache
        memcache_servers = 172.16.64.60:11211

        [filter:catch_errors]
        use = egg:swift#catch_errors

    Examples:
        >>> 'filter:cache' in object_expirer_conf
        True
        >>> object_expirer_conf.get('filter:cache', 'memcache_servers') == '172.16.64.60:11211'
        True
        >>> object_expirer_conf.getint('object-expirer', 'report_interval')
        300
    """
    pass


@parser(Specs.swift_conf)
class SwiftConf(IniConfigFile):
    """
    This class is to parse the content of ``/etc/swift/swift.conf``.

    ``/etc/swift/swift.conf`` is in the standard 'ini' format and is
    read by the :py:class:`insights.core.IniConfigFile` parser class.

    Sample configuration file::

        [swift-hash]
        # random unique strings that can never change (DO NOT LOSE)
        # Use only printable chars (python -c "import string; print(string.printable)")
        swift_hash_path_prefix = changeme
        swift_hash_path_suffix = changeme

        [storage-policy:0]
        name = gold
        policy_type = replication
        default = yes

        [storage-policy:1]
        name = silver
        policy_type = replication

        [storage-policy:2]
        name = ec42
        policy_type = erasure_coding
        ec_type = liberasurecode_rs_vand
        ec_num_data_fragments = 4
        ec_num_parity_fragments = 2

    Examples:
        >>> 'swift-hash' in swift_conf.sections()
        True
        >>> swift_conf.has_option('storage-policy:2', 'policy_type') is True
        True
        >>> swift_conf.get('storage-policy:2', 'policy_type') == 'erasure_coding'
        True
        >>> swift_conf.get('storage-policy:2', 'ec_type') == 'liberasurecode_rs_vand'
        True
    """
    pass



from __future__ import print_function
from __future__ import absolute_import
import sys
import json
import logging
import logging.handlers
import os
import time
import shutil
import six
import atexit

from .utilities import (generate_machine_id,
                        write_to_disk,
                        write_registered_file,
                        write_unregistered_file,
                        delete_registered_file,
                        delete_unregistered_file,
                        determine_hostname)
from .collection_rules import InsightsUploadConf
from .data_collector import DataCollector
from .connection import InsightsConnection
from .archive import InsightsArchive
from .support import registration_check
from .constants import InsightsConstants as constants
from .schedule import get_scheduler

LOG_FORMAT = ("%(asctime)s %(levelname)8s %(name)s %(message)s")
logger = logging.getLogger(__name__)


def do_log_rotation():
    handler = get_file_handler()
    return handler.doRollover()


def get_file_handler(config):
    log_file = config.logging_file
    log_dir = os.path.dirname(log_file)
    if not os.path.exists(log_dir):
        os.makedirs(log_dir, 0o700)
    file_handler = logging.handlers.RotatingFileHandler(
        log_file, backupCount=3)
    file_handler.setFormatter(logging.Formatter(LOG_FORMAT))
    return file_handler


def get_console_handler(config):
    if config.silent:
        target_level = logging.FATAL
    elif config.verbose:
        target_level = logging.DEBUG
    elif config.quiet:
        target_level = logging.ERROR
    else:
        target_level = logging.INFO

    handler = logging.StreamHandler(sys.stderr)
    handler.setLevel(target_level)

    log_format = LOG_FORMAT if config.verbose else "%(message)s"
    handler.setFormatter(logging.Formatter(log_format))

    return handler


def configure_level(config):
    config_level = 'DEBUG' if config.verbose else config.loglevel

    init_log_level = logging.getLevelName(config_level)
    if type(init_log_level) in six.string_types:
        print("Invalid log level %s, defaulting to DEBUG" % config_level)
        init_log_level = logging.DEBUG

    logger.setLevel(init_log_level)
    logging.root.setLevel(init_log_level)

    net_debug_level = logging.INFO if config.net_debug else logging.ERROR
    logging.getLogger('network').setLevel(net_debug_level)
    if not config.verbose:
        logging.getLogger('insights.core.dr').setLevel(logging.WARNING)


def set_up_logging(config):
    if len(logging.root.handlers) == 0:
        logging.root.addHandler(get_console_handler(config))
        logging.root.addHandler(get_file_handler(config))
        configure_level(config)
        logger.debug("Logging initialized")


# -LEGACY-
def register(config, pconn):
    """
    Do registration using basic auth
    """
    username = config.username
    password = config.password
    authmethod = config.authmethod
    auto_config = config.auto_config
    if not username and not password and not auto_config and authmethod == 'BASIC':
        logger.debug('Username and password must be defined in configuration file with BASIC authentication method.')
        return False
    return pconn.register()


# -LEGACY-
def _legacy_handle_registration(config, pconn):
    '''
    Handle the registration process
    Returns:
        True - machine is registered
        False - machine is unregistered
        None - could not reach the API
    '''
    logger.debug('Trying registration.')
    # force-reregister -- remove machine-id files and registration files
    # before trying to register again
    if config.reregister:
        delete_registered_file()
        delete_unregistered_file()
        write_to_disk(constants.machine_id_file, delete=True)
        logger.debug('Re-register set, forcing registration.')

    logger.debug('Machine-id: %s', generate_machine_id(new=config.reregister))

    # check registration with API
    check = get_registration_status(config, pconn)

    for m in check['messages']:
        logger.debug(m)

    if check['unreachable']:
        # Run connection test and exit
        return None

    if check['status']:
        # registered in API, resync files
        if config.register:
            logger.info('This host has already been registered.')
        write_registered_file()
        return True

    if config.register:
        if config.group:
            logger.info('Registering with a group is currently unavailable.')
        # register if specified
        message, hostname, group, display_name = register(config, pconn)
        if not hostname:
            # API could not be reached, run connection test and exit
            logger.error(message)
            return None
        if config.display_name is None and config.group is None:
            logger.info('Successfully registered host %s', hostname)
        elif config.display_name is None:
            logger.info('Successfully registered host %s in group %s',
                        hostname, group)
        else:
            logger.info('Successfully registered host %s as %s in group %s',
                        hostname, display_name, group)
        if message:
            logger.info(message)
        write_registered_file()
        return True
    else:
        # unregistered in API, resync files
        write_unregistered_file(date=check['unreg_date'])
        # print messaging and exit
        if check['unreg_date']:
            # registered and then unregistered
            logger.info('This machine has been unregistered. '
                        'Use --register if you would like to '
                        're-register this machine.')
        else:
            # not yet registered
            logger.info('This machine has not yet been registered.'
                        'Use --register to register this machine.')
        return False


def handle_registration(config, pconn):
    '''
    Does nothing on the platform. Will be deleted eventually.
    '''
    if config.legacy_upload:
        return _legacy_handle_registration(config, pconn)


def get_registration_status(config, pconn):
    '''
        Handle the registration process
        Returns:
            True - machine is registered
            False - machine is unregistered
            None - could not reach the API
    '''
    return registration_check(pconn)


# -LEGACY-
def _legacy_handle_unregistration(config, pconn):
    """
        returns (bool): True success, False failure
    """
    check = get_registration_status(config, pconn)

    for m in check['messages']:
        logger.debug(m)

    if check['unreachable']:
        # Run connection test and exit
        return None

    if check['status']:
        unreg = pconn.unregister()
    else:
        unreg = True
        logger.info('This system is already unregistered.')
    if unreg:
        # only set if unreg was successful
        write_unregistered_file()
        get_scheduler(config).remove_scheduling()
    return unreg


def handle_unregistration(config, pconn):
    """
    Returns:
        True - machine was successfully unregistered
        False - machine could not be unregistered
        None - could not reach the API
    """
    if config.legacy_upload:
        return _legacy_handle_unregistration(config, pconn)
    logger.info('Unregistration is not currently available.')
    return False
    # unreg = False  # pconn.unregister()
    # if unreg:
    #     # only set if unreg was successful
    #     write_unregistered_file()
    #     get_scheduler(config).remove_scheduling()
    # return unreg


def get_machine_id():
    return generate_machine_id()


def update_rules(config, pconn):
    if not pconn:
        raise ValueError('ERROR: Cannot update rules in --offline mode. '
                         'Disable auto_update in config file.')

    pc = InsightsUploadConf(config, conn=pconn)
    return pc.get_conf_update()


def get_branch_info(config):
    """
    Get branch info for a system
    returns (dict): {'remote_branch': -1, 'remote_leaf': -1}
    """
    # in the case we are running on offline mode
    # or we are analyzing a running container/image
    # or tar file, mountpoint, simply return the default branch info
    if (config.offline or
            config.analyze_container):
        return constants.default_branch_info
    return config.branch_info


def collect(config, pconn):
    """
    All the heavy lifting done here
    """
    # initialize collection target
    # tar files
    if config.analyze_file:
        logger.debug("Client analyzing a compress filesystem.")
        target = {'type': 'compressed_file',
                  'name': os.path.splitext(
                        os.path.basename(config.analyze_file))[0],
                  'location': config.analyze_file}

    # mountpoints
    elif config.analyze_mountpoint:
        logger.debug("Client analyzing a filesystem already mounted.")
        target = {'type': 'mountpoint',
                  'name': os.path.splitext(
                        os.path.basename(config.analyze_mountpoint))[0],
                  'location': config.analyze_mountpoint}

    # image
    elif config.analyze_image_id:
        logger.debug("Client running in image mode.")
        logger.debug("Scanning for matching image.")

        from .containers import get_targets
        targets = get_targets(config)
        if len(targets) == 0:
            sys.exit(constants.sig_kill_bad)
        target = targets[0]

    # host, or inside container
    else:
        if config.analyze_container:
            logger.debug('Client running in container mode.')
        else:
            logger.debug("Host selected as scanning target.")
        target = constants.default_target

    branch_info = get_branch_info(config)
    pc = InsightsUploadConf(config)
    tar_file = None

    collection_rules = pc.get_conf_file()
    rm_conf = pc.get_rm_conf()
    if rm_conf:
        logger.warn("WARNING: Excluding data from files")

    # defaults
    archive = None
    container_connection = None
    mp = None
    compressed_filesystem = None

    try:
        # analyze docker images
        if target['type'] == 'docker_image':
            from .containers import open_image
            container_connection = open_image(target['name'])
            logging_name = 'Docker image ' + target['name']

            if container_connection:
                mp = container_connection.get_fs()
            else:
                logger.error('Could not open %s for analysis', logging_name)
                return False

        # analyze compressed files
        elif target['type'] == 'compressed_file':
            logging_name = 'Compressed file ' + target['name'] + ' at location ' + target['location']

            from .compressed_file import InsightsCompressedFile
            compressed_filesystem = InsightsCompressedFile(target['location'])

            if compressed_filesystem.is_tarfile is False:
                logger.debug("Could not access compressed tar filesystem.")
                return False

            mp = compressed_filesystem.get_filesystem_path()

        # analyze mountpoints
        elif target['type'] == 'mountpoint':

            logging_name = 'Filesystem ' + target['name'] + ' at location ' + target['location']
            mp = config.analyze_mountpoint

        # analyze the host
        elif target['type'] == 'host':
            logging_name = determine_hostname()

        # nothing found to analyze
        else:
            logger.error('Unexpected analysis target: %s', target['type'])
            return False

        archive = InsightsArchive(compressor=config.compressor,
                                  target_name=target['name'])
        atexit.register(_delete_archive_internal, config, archive)

        # determine the target type and begin collection
        # we infer "docker_image" SPEC analysis for certain types
        if target['type'] in ["mountpoint", "compressed_file"]:
            target_type = "docker_image"
        else:
            target_type = target['type']
        logger.debug("Inferring target_type '%s' for SPEC collection", target_type)
        logger.debug("Inferred from '%s'", target['type'])
        dc = DataCollector(config, archive, mountpoint=mp)

        logger.info('Starting to collect Insights data for %s', logging_name)
        dc.run_collection(collection_rules, rm_conf, branch_info)

        tar_file = dc.done(collection_rules, rm_conf)

    finally:
        # called on loop iter end or unexpected exit
        if container_connection:
            container_connection.close()

    # cleanup the temporary stuff for analyzing tar files
    if config.analyze_file is not None and compressed_filesystem is not None:
        compressed_filesystem.cleanup_temp_filesystem()

    return tar_file


def get_connection(config):
    return InsightsConnection(config)


def _legacy_upload(config, pconn, tar_file, content_type, collection_duration=None):
    logger.info('Uploading Insights data.')
    api_response = None
    for tries in range(config.retries):
        upload = pconn.upload_archive(tar_file, '', collection_duration)

        if upload.status_code in (200, 201):
            api_response = json.loads(upload.text)

            # Write to last upload file
            with open(constants.last_upload_results_file, 'w') as handler:
                if six.PY3:
                    handler.write(upload.text)
                else:
                    handler.write(upload.text.encode('utf-8'))
            write_to_disk(constants.lastupload_file)

            msg_name = determine_hostname(config.display_name)
            account_number = config.account_number
            if account_number:
                logger.info("Successfully uploaded report from %s to account %s.",
                            msg_name, account_number)
            else:
                logger.info("Successfully uploaded report for %s.", msg_name)
            break

        elif upload.status_code == 412:
            pconn.handle_fail_rcs(upload)
            break
        else:
            logger.error("Upload attempt %d of %d failed! Status Code: %s",
                         tries + 1, config.retries, upload.status_code)
            if tries + 1 != config.retries:
                logger.info("Waiting %d seconds then retrying",
                            constants.sleep_time)
                time.sleep(constants.sleep_time)
            else:
                logger.error("All attempts to upload have failed!")
                logger.error("Please see %s for additional information", config.logging_file)
    return api_response


def upload(config, pconn, tar_file, content_type, collection_duration=None):
    if config.legacy_upload:
        return _legacy_upload(config, pconn, tar_file, content_type, collection_duration)
    logger.info('Uploading Insights data.')

    for tries in range(config.retries):
        upload = pconn.upload_archive(tar_file, content_type, collection_duration)

        if upload.status_code in (200, 202):
            msg_name = determine_hostname(config.display_name)
            logger.info("Successfully uploaded report for %s.", msg_name)
        else:
            logger.error("Upload attempt %d of %d failed! Status code: %s",
                         tries + 1, config.retries, upload.status_code)
            if tries + 1 != config.retries:
                logger.info("Waiting %d seconds then retrying",
                            constants.sleep_time)
                time.sleep(constants.sleep_time)
            else:
                logger.error("All attempts to upload have failed!")
                logger.error("Please see %s for additional information", config.logging_file)


def _delete_archive_internal(config, archive):
    '''
    Only used during built-in collection.
    Delete archive and tmp dirs on unexpected exit.
    '''
    if not config.keep_archive:
        archive.delete_tmp_dir()
        archive.delete_archive_file()


def delete_archive(path, delete_parent_dir):
    removed_archive = False

    try:
        logger.debug("Removing archive %s", path)
        removed_archive = os.remove(path)

        dirname = os.path.dirname
        abspath = os.path.abspath
        parent_tmp_dir = dirname(abspath(path))
        if delete_parent_dir:
            logger.debug("Detected parent temporary directory %s", parent_tmp_dir)
            if parent_tmp_dir != "/var/tmp" and parent_tmp_dir != "/var/tmp/":
                logger.debug("Removing %s", parent_tmp_dir)
                shutil.rmtree(parent_tmp_dir)

    except:
        logger.error("Error removing %s", path)

    return removed_archive



from insights.tests import context_wrap
from insights.parsers.grub_conf import Grub1Config, Grub2Config, Grub1EFIConfig
import pytest

# RHEL7
GRUB2_CFG_1 = """
if [ -s $prefix/grubenv ]; then
  load_env
fi
#[...]
if [ x"${feature_menuentry_id}" = xy ]; then
  menuentry_id_option="--id"
else
  menuentry_id_option=""
fi
## BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server, with Linux 3.10.0-123.9.3.el7.x86_64' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.9.3.el7.x86_64-advanced-5a1c841f-5cfe-4d59-b3a0-4b788369d6cb'
   {load_video
    set gfxpayload=keep
    insmod part_msdos
    set root='hd0,msdos1'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  93f2c0dc-6201-4d26-a610-4b3998983ea2
      if [ -s $prefix/grubenv ]; then
        load_env
      fi
    else
      search --no-floppy --fs-uuid --set=root 93f2c0dc-6201-4d26-a610-4b3998983ea2
    fi
    linux16 /vmlinuz-3.10.0-123.9.3.el7.x86_64 root=UUID=5a1c841f-5cfe-4d59-b3a0-4b788369d6cb ro crashkernel=auto  vconsole.font=latarcyrheb-sun16 console=ttyS0,38400 rd.lvm.lv=VG_RACS-CCP/lv-rootfs vconsole.keymap=us LANG=en_US.UTF-8
    initrd16 /initramfs-3.10.0-123.9.3.el7.x86_64.img
    insmod gzio}
menuentry 'Red Hat Enterprise Linux Server, with Linux 0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3-advanced-5a1c841f-5cfe-4d59-b3a0-4b788369d6cb'
    {
    insmod part_msdos
    insmod ext2
    set root='hd0,msdos1'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  93f2c0dc-6201-4d26-a610-4b3998983ea2
    else
      search --no-floppy --fs-uuid --set=root 93f2c0dc-6201-4d26-a610-4b3998983ea2
    fi
    linux16 /vmlinuz-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3 root=UUID=5a1c841f-5cfe-4d59-b3a0-4b788369d6cb ro crashkernel=auto  vconsole.font=latarcyrheb-sun16 console=ttyS0,38400 rd.lvm.lv=VG_RACS-CCP/lv-rootfs vconsole.keymap=us
    initrd16 /initramfs-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3.img
}
""".strip()

# RHEL7
GRUB2_CFG_2 = """
### BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server (3.10.0-229.el7.x86_64) 7.0 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.el7.x86_64-advanced-7f18fec3-a016-42ab-9bb9-6e7f6a5985ca' {
    linux16 /vmlinuz-3.10.0-229.el7.x86_64 root=/dev/mapper/rhel-root ro rd.lvm.lv=rhel/root crashkernel=auto  rd.lvm.lv=rhel/swap vconsole.font=latarcyrheb-sun16 vconsole.keymap=us rhgb quiet LANG=en_AU.UTF-8
    initrd16 /initramfs-3.10.0-229.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (3.10.0-123.13.2.el7.x86_64) 7.0 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.el7.x86_64-advanced-7f18fec3-a016-42ab-9bb9-6e7f6a5985ca' {
    linux16 /vmlinuz-3.10.0-123.13.2.el7.x86_64 root=/dev/mapper/rhel-root ro rd.lvm.lv=rhel/root crashkernel=auto  rd.lvm.lv=rhel/swap vconsole.font=latarcyrheb-sun16 vconsole.keymap=us rhgb quiet LANG=en_AU.UTF-8
    initrd16 /initramfs-3.10.0-123.13.2.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server, with Linux 3.10.0-123.el7.x86_64' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.el7.x86_64-advanced-7f18fec3-a016-42ab-9bb9-6e7f6a5985ca' {
    linux16 /vmlinuz-3.10.0-123.el7.x86_64 root=UUID=7f18fec3-a016-42ab-9bb9-6e7f6a5985ca ro rd.lvm.lv=rhel/root crashkernel=auto  rd.lvm.lv=rhel/swap vconsole.font=latarcyrheb-sun16 vconsole.keymap=us rhgb quiet LANG=en_AU.UTF-8
    initrd16 /initramfs-3.10.0-123.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server, with Linux 0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad-advanced-7f18fec3-a016-42ab-9bb9-6e7f6a5985ca' {
    linux16 /vmlinuz-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad root=UUID=7f18fec3-a016-42ab-9bb9-6e7f6a5985ca ro rd.lvm.lv=rhel/root crashkernel=auto  rd.lvm.lv=rhel/swap vconsole.font=latarcyrheb-sun16 vconsole.keymap=us rhgb quiet
    initrd16 /initramfs-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad.img
}
""".strip()

# RHEL5/6
GRUB1_CONF_3 = """
# grub.conf generated by anaconda
#
# Note that you do not have to rerun grub after making changes to this file
# NOTICE:  You have a /boot partition.  This means that
#          all kernel and initrd paths are relative to /boot/, eg.
#          root (hd0,0)
#          kernel /vmlinuz-version ro root=/dev/cciss/c0d0p3
#          initrd /initrd-version.img
#boot=/dev/cciss/c0d0
default=1
# fallback=0 # commented out by Proliant HBA install script
timeout=5
splashimage=(hd0,0)/grub/splash.xpm.gz
hiddenmenu
title Red Hat Enterprise Linux Server (2.6.18-194.8.1.el5)
    root (hd0,0)
    kernel /vmlinuz-2.6.18-194.8.1.el5 ro root=LABEL=/1 crashkernel=128M@16M
    initrd /initrd-2.6.18-194.8.1.el5.img

title Red Hat Enterprise Linux Server (2.6.18-194.17.1.el5)
    root (hd0,0)
    kernel /vmlinuz-2.6.18-194.17.1.el5 ro root=LABEL=/1 crashkernel=128M@16M
    module /initramfs-2.6.18-194.8.1.el5.img
""".strip()

# Incorrect `initrd` and `default`
GRUB1_CONF_4 = """
default=a
title (2.6.18-194.8.1.el5)
    kernel /vmlinuz-2.6.18-194.8.1.el5 ro root=LABEL=/1 crashkernel=128M@16M
    module /2.6.18-194.8.1.el5.img
""".strip()

# Empty kernel
GRUB1_CONF_5 = """
defaults=0
title (2.6.18-194.8.1.el5)
    kernel
    module /2.6.18-194.8.1.el5.img
""".strip()

# no configs
GRUB1_CONF_6 = """
title Red Hat Enterprise Linux Server
    kernel test
    module /2.6.18-194.8.1.el5.img
""".strip()

# too big default
GRUB1_CONF_7 = """
default = 1
title Red Hat Enterprise Linux Server (2.6.18-194.8.1.el5)
    kernel test
    module /2.6.18-194.8.1.el5.img
""".strip()

# only title
GRUB1_CONF_8 = """
title Red Hat Enterprise Linux Server (2.6.18-194.8.1.el5)
""".strip()

GRUB2_CFG_3 = """
menuentry 'Red Hat Enterprise Linux Server, with Linux 3.10.0-123.9.3.el7.x86_64' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.9.3.el7.x86_64-advanced-5a1c841f-5cfe-4d59-b3a0-4b788369d6cb' { load_video
    set gfxpayload=keep
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  93f2c0dc-6201-4d26-a610-4b3998983ea2
    else
      search --no-floppy --fs-uuid --set=root 93f2c0dc-6201-4d26-a610-4b3998983ea2
    fi
    linux16 /vmlinuz-3.10.0-123.9.3.el7.x86_64 root=UUID=5a1c841f-5cfe-4d59-b3a0-4b788369d6cb ro crashkernel=auto  vconsole.font=latarcyrheb-sun16 console=ttyS0,38400 rd.lvm.lv=VG_RACS-CCP/lv-rootfs vconsole.keymap=us LANG=en_US.UTF-8
    initrd16 /initramfs-3.10.0-123.9.3.el7.x86_64.img
}
""".strip()

GRUB2_CFG_4 = """
menuentry {
    linux16 /vmlinuz-3.10.0-123.9.3.el7.x86_64 crashkernel=auto
    initrd16 /initramfs-3.10.0-123.9.3.el7.x86_64.img
}
""".strip()


def test_grub_conf():
    expected_result = {'grub_kernels': ["vmlinuz-2.6.18-194.8.1.el5", "vmlinuz-2.6.18-194.17.1.el5"],
                       'grub_initrds': ["initrd-2.6.18-194.8.1.el5.img", "initramfs-2.6.18-194.8.1.el5.img"]}
    assert expected_result == Grub1Config(context_wrap(GRUB1_CONF_3)).kernel_initrds

    expected_result = {'grub_kernels': ["vmlinuz-2.6.18-194.8.1.el5"],
                       'grub_initrds': []}

    grub1 = Grub1Config(context_wrap(GRUB1_CONF_4))
    assert grub1.is_kdump_iommu_enabled is False
    assert expected_result == grub1.kernel_initrds
    assert grub1.get_current_title() is None

    grub1 = Grub1Config(context_wrap(GRUB1_CONF_5))
    assert grub1.is_kdump_iommu_enabled is False
    assert grub1.get_current_title() == [
        ('title_name', '(2.6.18-194.8.1.el5)'),
        ('kernel', None), ('module', '/2.6.18-194.8.1.el5.img')]

    grub1 = Grub1Config(context_wrap(GRUB1_CONF_6))
    assert grub1.is_kdump_iommu_enabled is False
    assert grub1.get_current_title() == [
        ('title_name', 'Red Hat Enterprise Linux Server'),
        ('kernel', 'test'), ('module', '/2.6.18-194.8.1.el5.img')]

    grub1 = Grub1Config(context_wrap(GRUB1_CONF_7))
    assert grub1.is_kdump_iommu_enabled is False
    assert grub1.get_current_title() is None

    grub1 = Grub1Config(context_wrap(GRUB1_CONF_8))
    assert grub1.is_kdump_iommu_enabled is False

    grub1efi = Grub1EFIConfig(context_wrap(GRUB1_CONF_4))
    assert grub1efi.get_current_title() is None

    grub1efi = Grub1EFIConfig(context_wrap(GRUB1_CONF_5))
    assert grub1efi.get_current_title() == [
        ('title_name', '(2.6.18-194.8.1.el5)'),
        ('kernel', None), ('module', '/2.6.18-194.8.1.el5.img')]

    grub1efi = Grub1EFIConfig(context_wrap(GRUB1_CONF_6))
    assert grub1efi.get_current_title() == [
        ('title_name', 'Red Hat Enterprise Linux Server'),
        ('kernel', 'test'), ('module', '/2.6.18-194.8.1.el5.img')]

    grub1efi = Grub1EFIConfig(context_wrap(GRUB1_CONF_7))
    assert grub1efi.get_current_title() is None

    grub_conf = Grub2Config(context_wrap(GRUB2_CFG_1))['menuentry']
    assert ('load_video', None) in grub_conf[0]
    assert ('load_env', None) not in grub_conf[0]
    assert ('insmod', 'gzio') in grub_conf[0]

    expected_result = {'grub_kernels': ["vmlinuz-3.10.0-229.el7.x86_64", "vmlinuz-3.10.0-123.13.2.el7.x86_64",
                                        "vmlinuz-3.10.0-123.el7.x86_64", "vmlinuz-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad"],
                       'grub_initrds': ["initramfs-3.10.0-229.el7.x86_64.img", "initramfs-3.10.0-123.13.2.el7.x86_64.img",
                                        "initramfs-3.10.0-123.el7.x86_64.img", "initramfs-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad.img"]}
    assert expected_result == Grub2Config(context_wrap(GRUB2_CFG_2)).kernel_initrds

    grub_conf = Grub2Config(context_wrap(GRUB2_CFG_3))
    assert ('load_video', None) in grub_conf['menuentry'][0]
    assert grub_conf.is_kdump_iommu_enabled is False


def test_grub_conf_raise():
    with pytest.raises(Exception) as e:
        Grub2Config(context_wrap(GRUB2_CFG_4))
    assert "Cannot parse menuentry line: menuentry {" in str(e)



"""
DockerInspect - Command ``docker inspect --type={TYPE}``
========================================================

This module parses the output of the ``docker inspect`` command.  This uses
the ``core.marshalling.unmarshal`` function to parse the JSON output from the
commands.  The data is stored as a nested dictionary in the ``data``
attribute, and uses the ``LegacyItemAccess`` mixin class to provide ``get``
and ``contains`` access to the dictionary via the object itself.

Sample input::

    [
    {
        "Id": "97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07",
        "Created": "2016-06-23T05:12:25.433469799Z",
        "Path": "/bin/bash",
        "Args": [],
        "Name": "/sample_webapp",
        "State": {
            "Status": "running",
            "Running": true,
            "Paused": false,
            "Restarting": false,
    ...

Examples:

    >>> image = shared[DockerInspectContainer]
    >>> image.data['ID'] # new-style access
    '97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07'
    >>> image['Name'] # old-style access
    '/sample_webapp'
    >>> image.get('State').get('Paused') # sub-dictionaries
    False

"""

from .. import LegacyItemAccess, parser, CommandParser
from ..core.marshalling import unmarshal
from insights.specs import Specs


class DockerInspect(LegacyItemAccess, CommandParser):
    """
    Parse the output of command "docker inspect --type=image" and "docker
    inspect --type=container".  The output of these two commands is formatted
    as JSON, so "json.loads" is an option to parse the output in the future.
    """

    def parse_content(self, content):
        content = "\n".join(list(content))
        try:
            inspect_data = unmarshal(content)
            self.data = inspect_data[0]
        except:
            self.data = {}


@parser(Specs.docker_image_inspect)
class DockerInspectImage(DockerInspect):
    """
    Parse docker image inspect output using the DockerInspect parser class.
    """
    pass


@parser(Specs.docker_container_inspect)
class DockerInspectContainer(DockerInspect):
    """
    Parse docker container inspect output using the DockerInspect parser class.
    """
    pass



# !/usr/bin/python
# Copyright (C) 2015 Red Hat, All rights reserved.
# AUTHORS: Alex Collins <alcollin@redhat.com>

import sys
import json
import subprocess
import collections
from fnmatch import fnmatch as matches
from docker.utils import kwargs_from_env

import docker
import selinux

"""Atomic Utility Module"""

ReturnTuple = collections.namedtuple('ReturnTuple',
                                     ['return_code', 'stdout', 'stderr'])


def _decompose(compound_name):
    """ '[reg/]repo[:tag]' -> (reg, repo, tag) """
    reg, repo, tag = '', compound_name, ''
    if '/' in repo:
        reg, repo = repo.split('/', 1)
    if ':' in repo:
        repo, tag = repo.rsplit(':', 1)
    return reg, repo, tag


def image_by_name(img_name, images=None):
    """
    Returns a list of image data for images which match img_name. Will
    optionally take a list of images from a docker.Client.images
    query to avoid multiple docker queries.
    """
    i_reg, i_rep, i_tag = _decompose(img_name)

    # Correct for bash-style matching expressions.
    if not i_reg:
        i_reg = '*'
    if not i_tag:
        i_tag = '*'

    # If the images were not passed in, go get them.
    if images is None:
        c = docker.Client(**kwargs_from_env())
        images = c.images(all=False)

    valid_images = []
    for i in images:
        for t in i['RepoTags']:
            reg, rep, tag = _decompose(t)
            if matches(reg, i_reg) \
                    and matches(rep, i_rep) \
                    and matches(tag, i_tag):
                valid_images.append(i)
                break
            # Some repo after decompose end up with the img_name
            # at the end.  i.e. rhel7/rsyslog
            if rep.endswith(img_name):
                valid_images.append(i)
                break
    return valid_images


def subp(cmd):
    """
    Run a command as a subprocess.
    Return a triple of return code, standard out, standard err.
    """
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    out, err = proc.communicate()
    return ReturnTuple(proc.returncode, stdout=out, stderr=err)


def default_container_context():
    if selinux.is_selinux_enabled() != 0:
        fd = open(selinux.selinux_lxc_contexts_path())
        for i in fd.readlines():
            name, context = i.split("=")
            if name.strip() == "file":
                return context.strip("\n\" ")
    return ""


def writeOut(output, lf="\n"):
    sys.stdout.flush()
    sys.stdout.write(str(output) + lf)


def output_json(json_data):
    ''' Pretty print json data '''
    writeOut(json.dumps(json_data, indent=4, separators=(',', ': ')))


def print_scan_summary(json_data, names=None):
    '''
    Print a summary of the data returned from a
    CVE scan.
    '''
    max_col_width = 50
    min_width = 15

    def _max_width(data):
        max_name = 0
        for name in data:
            max_name = len(data[name]) if len(data[name]) > max_name \
                else max_name
        # If the max name length is less that max_width
        if max_name < min_width:
            max_name = min_width

        # If the man name is greater than the max col leng
        # we wish to use
        if max_name > max_col_width:
            max_name = max_col_width

        return max_name

    clean = True

    if len(names) > 0:
        max_width = _max_width(names)
    else:
        max_width = min_width
    template = "{0:" + str(max_width) + "}   {1:5} {2:5} {3:5} {4:5}"
    sevs = ['critical', 'important', 'moderate', 'low']
    writeOut(template.format("Container/Image", "Cri", "Imp", "Med", "Low"))
    writeOut(template.format("-" * max_width, "---", "---", "---", "---"))
    res_summary = json_data['results_summary']
    for image in res_summary.keys():
        image_res = res_summary[image]
        if 'msg' in image_res.keys():
            tmp_tuple = (image_res['msg'], "", "", "", "")
        else:
            if len(names) < 1:
                image_name = image[:max_width]
            else:
                image_name = names[image][-max_width:]
                if len(image_name) == max_col_width:
                    image_name = '...' + image_name[-(len(image_name) - 3):]

            tmp_tuple = tuple([image_name] +
                              [str(image_res[sev]) for sev in sevs])
            sev_results = [image_res[sev] for sev in
                           sevs if image_res[sev] > 0]
            if len(sev_results) > 0:
                clean = False
        writeOut(template.format(*tmp_tuple))
    writeOut("")
    return clean


def print_detail_scan_summary(json_data, names=None):
    '''
    Print a detailed summary of the data returned from
    a CVE scan.
    '''
    clean = True
    sevs = ['Critical', 'Important', 'Moderate', 'Low']
    cve_summary = json_data['host_results']
    image_template = "  {0:10}: {1}"
    cve_template = "     {0:10}: {1}"
    for image in cve_summary.keys():
        image_res = cve_summary[image]
        writeOut("")
        writeOut(image[:12])
        if not image_res['isRHEL']:
            writeOut(image_template.format("Result",
                                           "Not based on Red Hat"
                                           "Enterprise Linux"))
            continue
        else:
            writeOut(image_template.format("OS", image_res['os'].rstrip()))
            scan_results = image_res['cve_summary']['scan_results']

        for sev in sevs:
            if sev in scan_results:
                clean = False
                writeOut(image_template.format(sev,
                                               str(scan_results[sev]['num'])))
                for cve in scan_results[sev]['cves']:
                        writeOut(cve_template.format("CVE", cve['cve_title']))
                        writeOut(cve_template.format("CVE URL",
                                                     cve['cve_ref_url']))
                        writeOut(cve_template.format("RHSA ID",
                                                     cve['rhsa_ref_id']))
                        writeOut(cve_template.format("RHSA URL",
                                                     cve['rhsa_ref_url']))
                        writeOut("")
    return clean


def get_mounts_by_path():
    '''
    Gets all mounted devices and paths
    :return: dict of mounted devices and related information by path
    '''
    mount_info = []
    f = open('/proc/mounts', 'r')
    for line in f:
        _tmp = line.split(" ")
        mount_info.append({'path': _tmp[1],
                           'device': _tmp[0],
                           'type': _tmp[2],
                           'options': _tmp[3]
                           }
                          )
    return mount_info


def is_dock_obj_mounted(docker_obj):
    '''
    Check if the provided docker object, which needs to be an ID,
    is currently mounted and should be considered "busy"
    :param docker_obj: str, must be in ID format
    :return: bool True or False
    '''
    mount_info = get_mounts_by_path()
    devices = [x['device'] for x in mount_info]
    # If we can find the ID of the object in the list
    # of devices which comes from mount, safe to assume
    # it is busy.
    return any(docker_obj in x for x in devices)


def urllib3_disable_warnings():
    if 'requests' not in sys.modules:
        import requests
    else:
        requests = sys.modules['requests']

    # On latest Fedora, this is a symlink
    if hasattr(requests, 'packages'):
        requests.packages.urllib3.disable_warnings()  # pylint: disable=maybe-no-member
    else:
        # But with python-requests-2.4.3-1.el7.noarch, we need
        # to talk to urllib3 directly
        have_urllib3 = False
        try:
            if 'urllib3' not in sys.modules:
                import urllib3
                have_urllib3 = True
        except ImportError:
            pass
        if have_urllib3:
            # Except only call disable-warnings if it exists
            if hasattr(urllib3, 'disable_warnings'):
                urllib3.disable_warnings()



"""
GRUB configuration files
=======================================================================

This parser reads the configuration of the GRand Unified Bootloader, versions
1 or 2.

This is currently a fairly simple parsing process.  Data read from the file
is put into roughly three categories:

* **configs**: lines read from the file that aren't boot options (i.e.
  excluding lines that go in the *title* and *menuentry* sections).  These
  are split into pairs on the first '=' sign.
* **title**: (GRUB v1 only) lines prefixed by the word 'title'.  All following
  lines up to the next title line are folded together.
* **menuentry**: (GRUB v2 only) lines prefixed by the word 'menuentry'.  All
  following lines up to the line starting with '}' are treated as part of one
  menu entry.

Each of these categories is (currently) stored as a simple list of tuples.

* For the list of **configs**, the tuples are (key, value) pairs based on
  the line, split on the first '=' character.  If nothing is found after the
  '=' character, then the value is ``None``.

* For the **title** list, there will be exactly two items in this list:

  * The first item will be a tuple of two items: 'title_name' and the
    title of the boot option.
  * The second item will be a tuple of two items: 'kernel' and the entire
    rest of the kernel boot line as if it had been given all on one line.

* For the **menuentry** list:

  * the first item will be a tuple of two items: 'menuentry_name' and the
    full text between 'menuentry' and '{'.
  * the rest of the items will be tuples of that line in the menu entry
    configuration, split on the first space.  If no space is found after the
    first word, the value will be ``None``.  So ``load_video`` will be stored
    as ``('load_video', None)`` and ``set root='hd0,msdos1'`` will be stored
    as ``('set', "root='hd0,msdos1'")``.

Note:
    For GRUB version 2, all lines between the ``if`` and ``fi`` will be ignored
    due to we cannot analyze the result of the bash conditions.

There are several helper functions for dealing with the Intel IOMMU and for
extracting the kernel and initrd configurations available.

Parsers provided by this module are:

Grub1Config - file ``/boot/grub.conf``
--------------------------------------

Grub1EFIConfig - file ``/boot/efi/EFI/redhat/grub.conf``
--------------------------------------------------------

Grub2Config - file ``/boot/grub/grub2.cfg``
-------------------------------------------

Grub2EFIConfig - file ``/boot/efi/EFI/redhat/grub.cfg``
-------------------------------------------------------
"""

from .. import Parser, parser, get_active_lines, defaults, LegacyItemAccess
from insights.specs import Specs

IOMMU = "intel_iommu=on"
GRUB_KERNELS = 'grub_kernels'
GRUB_INITRDS = 'grub_initrds'


class BootEntry(LegacyItemAccess):
    """
    An object representing an entry in the output of ``mount`` command.  Each
    entry contains below fixed attributes:

    Attributes:
        name (str): Name of the boot entry
        cmdline (str): Cmdline of the boot entry
    """
    def __init__(self, data={}):
        self.data = data
        for k, v in data.items():
            setattr(self, k, v)

    def items(self):
        """
        To keep backward compatibility and let it can be iterated as a
        dictionary.
        """
        for k, v in self.data.items():
            yield k, v


class GrubConfig(LegacyItemAccess, Parser):
    """
    Parser for configuration for both GRUB versions 1 and 2.
    """

    def __init__(self, *args, **kwargs):
        self._boot_entries = []
        super(GrubConfig, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        """
        Parse grub configuration file to create a dict with this structure::

            {
                "configs": [ (name, value), (name, value) ...],
                "title": [
                    [(title_name, name), (cmd, opt), (cmd, opt) ...],
                    [(title_name, another_name), ...]
                ],
                "menuentry": [
                    [(menuentry_name, its name), (cmd, opt), (cmd, opt) ...],
                    [(menuentry_name, another_name), ...]
                ],
            }
        """

        line_iter = iter(get_active_lines(content))
        conf = {"configs": [], "title": [], "menuentry": []}
        line = None

        while (True):
            try:

                if line is None:
                    line = next(line_iter)

                if line.startswith('title '):
                    last_line = _parse_title(line_iter, line, conf)
                    line = last_line
                elif line.startswith('menuentry '):
                    _parse_menu_entry(line_iter, line, conf)
                    line = None
                else:
                    conf["configs"].append(_parse_config(line))
                    line = None

            except StopIteration:
                self.data = conf
                break

        if not self.data.get('title'):
            self.data.pop('title')
        if not self.data.get('menuentry'):
            self.data.pop('menuentry')
        if not self.data.get('configs'):
            self.data.pop('configs')

        for line_full in self.data.get('title', []) + self.data.get('menuentry', []):
            for name, line in line_full:
                if name == 'menuentry_name' or name == 'title_name':
                    entry = {}
                    entry['name'] = line
                elif entry and name.startswith(('kernel', 'linux')):
                    entry['cmdline'] = line
                    self._boot_entries.append(BootEntry(entry))

    @property
    def boot_entries(self):
        """
        Get all boot entries in GRUB configuration.

        Returns:
            (list): A list of :class:`BootEntry` objects for each boot entry in below format:
                    - 'name': "Red Hat Enterprise Linux Server"
                    - 'cmdline': "kernel /vmlinuz-2.6.32-431.11.2.el6.x86_64 crashkernel=128M rhgb quiet"
        """

        return self._boot_entries

    @property
    @defaults()
    def is_kdump_iommu_enabled(self):
        """
        Does any kernel have 'intel_iommu=on' set?

        Returns:
            (bool): ``True`` when 'intel_iommu=on' is set, otherwise returns ``False``
        """

        for line in self._boot_entries:
            if line.cmdline and IOMMU in line.cmdline:
                return True
        return False

    @property
    @defaults()
    def kernel_initrds(self):
        """
        Get the `kernel` and `initrd` files referenced in GRUB configuration files

        Returns:
            (dict): Returns a dict of the `kernel` and `initrd` files referenced
                    in GRUB configuration files
        """

        kernels = []
        initrds = []
        name_values = [(k, v) for k, v in self.data.get('configs', [])]
        for value in self.data.get('title', []) + self.data.get('menuentry', []):
            name_values.extend(value)

        for name, value in name_values:
            if name.startswith('module'):
                if 'vmlinuz' in value:
                    kernels.append(_parse_kernel_initrds_value(value))
                elif 'initrd' in value or 'initramfs' in value:
                    initrds.append(_parse_kernel_initrds_value(value))
            elif (name.startswith(('kernel', 'linux'))):
                if 'ipxe.lkrn' in value:
                    # Machine PXE boots the kernel, assume all is ok
                    return {}
                elif 'xen.gz' not in value:
                    kernels.append(_parse_kernel_initrds_value(value))
            elif name.startswith('initrd') or name.startswith('initrd16'):
                initrds.append(_parse_kernel_initrds_value(value))

        return {GRUB_KERNELS: kernels, GRUB_INITRDS: initrds}


@parser(Specs.grub_conf)
class Grub1Config(GrubConfig):
    """
    Parser for configuration for GRUB version 1.

    Examples:
        >>> grub1_content = '''
        ... default=0
        ... timeout=0
        ... splashimage=(hd0,0)/grub/splash.xpm.gz
        ... hiddenmenu
        ... title Red Hat Enterprise Linux Server (2.6.32-431.17.1.el6.x86_64)
        ...         kernel /vmlinuz-2.6.32-431.17.1.el6.x86_64 crashkernel=128M rhgb quiet
        ... title Red Hat Enterprise Linux Server (2.6.32-431.11.2.el6.x86_64)
        ...         kernel /vmlinuz-2.6.32-431.11.2.el6.x86_64 crashkernel=128M rhgb quiet
        ... '''.strip()

        >>> from insights.tests import context_wrap
        >>> shared = {Grub1Config: Grub1Config(context_wrap(grub1_content))}
        >>> config = shared[Grub1Config]
        >>> config['configs']
        [('default', '0'), ('timeout', '0'), ('splashimage', '(hd0,0)/grub/splash.xpm.gz'), ('hiddenmenu', None)]
        >>> config['title'][0]
        [('title_name', 'Red Hat Enterprise Linux Server (2.6.32-431.17.1.el6.x86_64)'), ('kernel', '/vmlinuz-2.6.32-431.17.1.el6.x86_64 crashkernel=128M rhgb quiet')]
        >>> config['title'][1][0][1]
        'Red Hat Enterprise Linux Server (2.6.32-431.11.2.el6.x86_64)'
        >>> config.boot_entries[1].name
        'Red Hat Enterprise Linux Server (2.6.32-431.11.2.el6.x86_64)'
        >>> config.boot_entries[1].cmdline
        "kernel /vmlinuz-2.6.32-431.11.2.el6.x86_64 crashkernel=128M rhgb quiet"
        >>> config.is_kdump_iommu_enabled
        False
        >>> config.kernel_initrds['grub_kernels'][0]
        'vmlinuz-2.6.32-431.17.1.el6.x86_64'
    """

    def __init__(self, *args, **kwargs):
        super(Grub1Config, self).__init__(*args, **kwargs)
        self._version = 1
        self._efi = False

    def get_current_title(self):
        """
        Get the current default title from the ``default`` option in the
        main configuration. (GRUB v1 only)

        Returns:
            list: A list contains all settings of the default boot entry:
                  - [(title_name, name), (cmd, opt), (cmd, opt) ...]
        """
        # if no 'default' in grub.conf, set default to 0
        idx = '0'
        conf = self.data.get('configs', [])
        for v in conf:
            if v[0] == 'default':
                idx = v[1]

        if idx.isdigit():
            idx = int(idx)
            title = self.data['title']
            if len(title) > idx:
                return title[idx]

        return None


@parser(Specs.grub_efi_conf)
class Grub1EFIConfig(Grub1Config):
    """
    Parses grub v1 configuration for EFI-based systems
    Content of grub-efi.conf is the same as grub.conf
    """
    def __init__(self, *args, **kwargs):
        super(Grub1EFIConfig, self).__init__(*args, **kwargs)
        self._efi = True


@parser(Specs.grub2_cfg)
class Grub2Config(GrubConfig):
    """
    Parser for configuration for GRUB version 2.

    Examples:
        >>> grub2_content = '''
        ... ### BEGIN /etc/grub.d/00_header ###
        ... set pager=1
        ... /
        ... if [ -s $prefix/grubenv ]; then
        ...   load_env
        ... fi
        ... #[...]
        ... if [ x"${feature_menuentry_id}" = xy ]; then
        ...   menuentry_id_option="--id"
        ... else
        ...   menuentry_id_option=""
        ... fi
        ... #[...]
        ... ### BEGIN /etc/grub.d/10_linux ###
        ...     menuentry 'Red Hat Enterprise Linux Workstation (3.10.0-327.36.3.el7.x86_64) 7.2 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.13.2.el7.x86_64-advanced-fbff9f50-62c3-484e-bca5-d53f672cda7c' {
        ...     load_video
        ...     set gfxpayload=keep
        ...     insmod gzio
        ...     insmod part_msdos
        ...     insmod ext2
        ...     set root='hd0,msdos1'
        ...     if [ x$feature_platform_search_hint = xy ]; then
        ...       search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  1184ab74-77b5-4cfa-81d3-fb87b0457577
        ...     else
        ...       search --no-floppy --fs-uuid --set=root 1184ab74-77b5-4cfa-81d3-fb87b0457577
        ...     fi
        ...     linux16 /vmlinuz-3.10.0-327.36.3.el7.x86_64 root=/dev/RHEL7CSB/Root ro rd.lvm.lv=RHEL7CSB/Root rd.luks.uuid=luks-96c66446-77fd-4431-9508-f6912bd84194 crashkernel=128M@16M rd.lvm.lv=RHEL7CSB/Swap vconsole.font=latarcyrheb-sun16 rhgb quiet LANG=en_GB.utf8
        ...     initrd16 /initramfs-3.10.0-327.36.3.el7.x86_64.img
        ...     }
        ... '''.strip()

        >>> from insights.tests import context_wrap
        >>> shared = {Grub2Config: Grub2Config(context_wrap(grub2_content))}
        >>> config = shared[Grub2Config]
        >>> config.boot_entries[0].name
        "'Red Hat Enterprise Linux Workstation (3.10.0-327.36.3.el7.x86_64) 7.2 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.13.2.el7.x86_64-advanced-fbff9f50-62c3-484e-bca5-d53f672cda7c'"
        >>> config.boot_entries[0].cmdline
        "linux16 /vmlinuz-3.10.0-327.36.3.el7.x86_64 root=/dev/RHEL7CSB/Root ro rd.lvm.lv=RHEL7CSB/Root rd.luks.uuid=luks-96c66446-77fd-4431-9508-f6912bd84194 crashkernel=128M@16M rd.lvm.lv=RHEL7CSB/Swap vconsole.font=latarcyrheb-sun16 rhgb quiet LANG=en_GB.utf8"
        >>> config['configs']
        [('set pager', '1'), ('/', None)]
        >>> config['menuentry']
        [[('menuentry_name', "'Red Hat Enterprise Linux Workstation (3.10.0-327.36.3.el7.x86_64) 7.2 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.13.2.el7.x86_64-advanced-fbff9f50-62c3-484e-bca5-d53f672cda7c'"), ('load_video', None), ('set', 'gfxpayload=keep'), ('insmod', 'gzio'), ('insmod', 'part_msdos'), ('insmod', 'ext2'), ('set', "root='hd0,msdos1'"), ('linux16', '/vmlinuz-3.10.0-327.36.3.el7.x86_64 root=/dev/RHEL7CSB/Root ro rd.lvm.lv=RHEL7CSB/Root rd.luks.uuid=luks-96c66446-77fd-4431-9508-f6912bd84194 crashkernel=128M@16M rd.lvm.lv=RHEL7CSB/Swap vconsole.font=latarcyrheb-sun16 rhgb quiet LANG=en_GB.utf8'), ('initrd16', '/initramfs-3.10.0-327.36.3.el7.x86_64.img')]]
        >>> config.kernel_initrds['grub_kernels'][0]
        'vmlinuz-3.10.0-327.36.3.el7.x86_64'
        >>> config.is_kdump_iommu_enabled
        False
    """
    def __init__(self, *args, **kwargs):
        super(Grub2Config, self).__init__(*args, **kwargs)
        self._version = 2
        self._efi = False


@parser(Specs.grub2_efi_cfg)
class Grub2EFIConfig(GrubConfig):
    """Parses grub2 configuration for EFI-based systems"""
    def __init__(self, *args, **kwargs):
        super(Grub2EFIConfig, self).__init__(*args, **kwargs)
        self._version = 2
        self._efi = True


def _parse_line(sep, line):
    """
    Parse a grub commands/config with format: cmd{sep}opts
    Returns: (name, value): value can be None
    """
    strs = line.split(sep, 1)
    return (strs[0].strip(), None) if len(strs) == 1 else (strs[0].strip(), strs[1].strip())


def _parse_cmd(line):
    """
    Parse commands within grub v1/v2 config using space delimeter
    """
    return _parse_line(" ", line)


def _parse_config(line):
    """
    Parse configuration lines in grub v1/v2 config
    """
    if "=" not in line:
        return _parse_cmd(line)
    else:
        return _parse_line("=", line)


def _parse_script(list, line, line_iter):
    """
    Eliminate any bash script contained in the grub v2 configuration
    """
    ifIdx = 0
    while (True):
        line = next(line_iter)
        if line.startswith("fi"):
            if ifIdx == 0:
                return
            ifIdx -= 1
        elif line.startswith("if"):
            ifIdx += 1


def _parse_menu_entry(line_iter, cur_line, conf):
    """
    Parse each `menuentry` that the grub v2 configuration contains
    * Uses "_parse_script" to eliminate bash scripts
    """
    menu = []
    conf['menuentry'].append(menu)
    n, entry = _parse_line("menuentry", cur_line)

    entry_name, v = _parse_line("{", entry)
    if not entry_name:
        raise Exception("Cannot parse menuentry line: {0}".format(cur_line))

    menu.append(('menuentry_name', entry_name))
    if v:
        menu.append(_parse_cmd(v))

    while (True):
        line = next(line_iter)
        if "{" in line:
            n, v = _parse_line("{", line)
            if v:
                menu.append(_parse_cmd(v))
        elif "}" in line:
            n, v = _parse_line("}", line)
            if n:
                menu.append(_parse_cmd(n))
            return
        elif line.startswith("if"):
            _parse_script(menu, line, line_iter)

        else:
            menu.append(_parse_cmd(line))


def _parse_title(line_iter, cur_line, conf):
    """
    Parse "title" in grub v1 config
    """
    title = []
    conf['title'].append(title)
    title.append(('title_name', cur_line.split('title', 1)[1].strip()))
    while (True):
        line = next(line_iter)
        if line.startswith("title "):
            return line

        cmd, opt = _parse_cmd(line)
        title.append((cmd, opt))


def _parse_kernel_initrds_value(line):
    """
    Called by "kernel_initrds" method to parse the kernel and
    initrds lines in the grub v1/v2 config
    """
    return line.split()[0].split('/')[-1]



"""
hostname - command ``/bin/hostname``
====================================

This parser simply reads the output of ``/bin/hostname``, which is the
configured fully qualified domain name of the client system.  It then
splits it into ``hostname`` and ``domain`` and stores these as attributes,
along with the unmodified name in the ``fqdn`` attribute.

Examples:

    >>> hostname = shared[Hostname]
    >>> hostname.fqdn
    'www.example.com'
    >>> hostname.hostname
    'www'
    >>> hostname.domain
    'example.com'

"""

from . import ParseException
from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.hostname)
class Hostname(CommandParser):
    """Class for parsing ``hostname`` command output.

    Attributes:
        fqdn: The fully qualified domain name of the host. The same to
            ``hostname`` when domain part is not set.
        hostname: The hostname.
        domain: The domain get from the fqdn.
    """
    def parse_content(self, content):
        content = list(filter(None, content))
        if len(content) != 1:
            msg = "hostname data contains multiple non-empty lines"
            raise ParseException(msg)
        raw = content[0].strip()
        self.fqdn = raw
        self.hostname = raw.split(".")[0] if raw else None
        self.domain = ".".join(raw.split(".")[1:]) if raw else None

    def __str__(self):
        return "<hostname: {h}, domain: {d}>".format(h=self.hostname, d=self.domain)



""" Test the query tool. """
import inspect

from insights import dr
from insights.tests import InputData
from insights.tools.query import load_obj, get_source, get_pydoc


def test_load_obj():
    assert load_obj("insights.dr") is dr
    assert load_obj("insights.tests.InputData") is InputData
    assert load_obj("foo.bar") is None


def test_get_source():
    assert get_source("insights.dr") == inspect.getsource(dr)
    assert get_source("insights.tests.InputData") == inspect.getsource(InputData)
    assert get_source("foo.bar") is None


def test_get_pydoc():
    assert get_pydoc("insights.dr") is not None
    assert get_pydoc("insights.tests.InputData") is not None
    assert get_pydoc("foo.bar") is None



""""
test mount
==========
"""
from insights.parsers import ParseException
from insights.parsers.mount import Mount
from insights.tests import context_wrap

import pytest

MOUNT_DATA = """
tmpfs on /tmp type tmpfs (rw,seclabel)
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
nfsd on /proc/fs/nfsd type nfsd (rw,relatime)
/dev/sda1 on /boot type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/fedora-home on /home type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/fedora-root on / type ext4 (rw,relatime,seclabel,data=ordered)
sunrpc on /var/lib/nfs/rpc_pipefs type rpc_pipefs (rw,relatime)
tmpfs on /run/user/42 type tmpfs (rw,nosuid,nodev,relatime,seclabel,size=1605428k,mode=700,uid=42,gid=42)
tmpfs on /run/user/1000 type tmpfs (rw,nosuid,nodev,relatime,seclabel,size=1605428k,mode=700,uid=1000,gid=1000)
gvfsd-fuse on /run/user/1000/gvfs type fuse.gvfsd-fuse (rw,nosuid,nodev,relatime,user_id=1000,group_id=1000)
fusectl on /sys/fs/fuse/connections type fusectl (rw,relatime)
/dev/mapper/HostVG-Config on /etc/shadow type ext4 (rw,noatime,seclabel,stripe=256,data=ordered) [CONFIG]
/dev/sr0 on /run/media/root/VMware Tools type iso9660 (ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2) [VMware Tools]
""".strip()

# Missing 'on' in second line
MOUNT_ERR_DATA = """
tmpfs on /tmp type tmpfs (rw,seclabel)
hugetlbfs /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
/dev/mapper/fedora-root on / type ext4 (rw,relatime,seclabel,data=ordered)
""".strip()


def test_mount():
    results = Mount(context_wrap(MOUNT_DATA))
    assert results is not None
    assert len(results) == 13
    sr0 = results.search(filesystem='/dev/sr0')[0]
    sda1 = results.search(filesystem='/dev/sda1')[0]
    assert sr0 is not None
    assert sr0['mount_point'] == '/run/media/root/VMware Tools'
    # test get method
    assert sr0.get('mount_point') == '/run/media/root/VMware Tools'
    assert sr0.get('does not exist', 'failure') == 'failure'
    assert sr0['mount_type'] == 'iso9660'
    assert 'ro' in sr0['mount_options']
    assert sr0.mount_options.ro
    assert 'relatime' in sr0['mount_options']
    assert sr0['mount_options']['uhelper'] == 'udisks2'
    assert sr0['mount_label'] == 'VMware Tools'
    assert sda1 is not None
    assert sda1['mount_point'] == '/boot'
    assert sda1['mount_type'] == 'ext4'
    assert 'rw' in sda1['mount_options']
    assert 'seclabel' in sda1['mount_options']
    assert sda1['mount_options']['data'] == 'ordered'
    assert sda1.mount_options.data == 'ordered'
    assert 'mount_label' not in sda1

    # Test iteration
    for mount in results:
        assert hasattr(mount, 'filesystem')
        assert hasattr(mount, 'mount_point')
        assert hasattr(mount, 'mount_type')
        assert hasattr(mount, 'mount_options')

    # Test getitem
    assert results[12] == sr0
    assert results['/etc/shadow'] == results[11]
    # Index only by string or number
    with pytest.raises(TypeError) as exc:
        assert results[set([1, 2, 3])] is None
    assert "Mounts can only be indexed by mount string or line number" in str(exc)

    # Test mounts dictionary
    assert results.mounts['/run/media/root/VMware Tools'] == sr0

    # Test get_dir
    assert results.get_dir('/run/media/root/VMware Tools') == sr0
    assert results.get_dir('/boot/grub2/grub.cfg') == sda1
    assert results.get_dir('/etc') == results['/']
    assert results.get_dir('relative/paths/fail') is None

    # Test parse failure
    errors = Mount(context_wrap(MOUNT_ERR_DATA))
    assert errors is not None
    assert len(errors) == 3
    assert not hasattr(errors[0], 'parse_error')
    assert errors[0].filesystem == 'tmpfs'
    assert hasattr(errors[1], 'parse_error')
    assert errors[1].parse_error == 'Unable to parse line'

    # Test search
    assert results.search(filesystem='/dev/sr0') == [sr0]
    assert results.search(mount_type='tmpfs') == [
        results.rows[n] for n in (0, 7, 8)
    ]
    assert results.search(mount_options__contains='seclabel') == [
        results.rows[n] for n in (0, 1, 3, 4, 5, 7, 8, 11)
    ]


MOUNT_WITHOUT_ROOT = """
tmpfs on /tmp type tmpfs (rw,seclabel)
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
/dev/sda1 on /boot type ext4 (rw,relatime,seclabel,data=ordered)
""".strip()


def test_mount_get_dir():
    with pytest.raises(ParseException) as exc:
        Mount(context_wrap(MOUNT_WITHOUT_ROOT))
    assert "Input for mount must contain '/' mount point." in str(exc)



"""
KeystoneConf - file ``/etc/keystone/keystone.conf``
===================================================

The ``KeystoneConf`` class parses the information in the file
``/etc/keystone/keystone.conf``.  See the ``IniConfigFile`` class for more
information on attributes and methods.

Sample input data looks like::

    [DEFAULT]

    #
    # From keystone
    #
    admin_token = ADMIN
    compute_port = 8774

    [identity]

    # From keystone
    default_domain_id = default
    #domain_specific_drivers_enabled = false
    domain_configurations_from_database = false

    [identity_mapping]

    driver = keystone.identity.mapping_backends.sql.Mapping
    generator = keystone.identity.id_generators.sha256.Generator
    #backward_compatible_ids = true

Examples:

    >>> kconf = shared[KeystoneConf]
    >>> kconf.defaults()
    {'admin_token': 'ADMIN', 'compute_port': '8774'}
    >>> 'identity' in kconf
    True
    >>> kconf.has_option('identity', 'default_domain_id')
    True
    >>> kconf.has_option('identity', 'domain_specific_drivers_enabled')
    False
    >>> kconf.get('identity', 'default_domain_id')
    'default'
    >>> kconf.items('identity_mapping')
    {'driver': 'keystone.identity.mapping_backends.sql.Mapping',
     'generator': 'keystone.identity.id_generators.sha256.Generator'}
"""
from .. import IniConfigFile, parser
from insights.specs import Specs


@parser(Specs.keystone_conf)
class KeystoneConf(IniConfigFile):
    """Parse contents of file ``/etc/keystone/keystone.conf``."""
    pass



from insights.parsers.hosts import Hosts
from insights.tests import context_wrap

HOSTS_EXAMPLE = """
127.0.0.1 localhost localhost.localdomain localhost4 localhost4.localdomain4
::1 localhost localhost.localdomain localhost6 localhost6.localdomain6

# Comment
127.0.0.1 fte.foo.redhat.com ci.foo.redhat.com qa.foo.redhat.com stage.foo.redhat.com prod.foo.redhat.com # Comment at end of line

10.0.0.1 nonlocal.foo.redhat.com nonlocal2.bar.redhat.com
2001:db8::36    ipv6.example.com

# Hosts with tabs
192.0.2.1	bar.example.com	bar
2001:db8::38	atlas	atlas.example.com
""".strip() + "                  "

EXPECTED = {
    "127.0.0.1": [
        "localhost",
        "localhost.localdomain",
        "localhost4",
        "localhost4.localdomain4",
        "fte.foo.redhat.com",
        "ci.foo.redhat.com",
        "qa.foo.redhat.com",
        "stage.foo.redhat.com",
        "prod.foo.redhat.com"
    ],
    "::1": [
        "localhost",
        "localhost.localdomain",
        "localhost6",
        "localhost6.localdomain6"
    ],
    "10.0.0.1": [
        "nonlocal.foo.redhat.com",
        "nonlocal2.bar.redhat.com"
    ],
    '2001:db8::36': ['ipv6.example.com'],
    '192.0.2.1': ['bar.example.com', 'bar'],
    '2001:db8::38': ['atlas', 'atlas.example.com'],
}


def test_hosts():
    d = Hosts(context_wrap(HOSTS_EXAMPLE)).data
    assert len(d) == 6
    for key in ["127.0.0.1", "::1"]:
        assert key in d
        assert d[key] == EXPECTED[key]


def test_all_hosts():
    all_names = Hosts(context_wrap(HOSTS_EXAMPLE)).all_names
    expected = set(EXPECTED["127.0.0.1"]) | set(EXPECTED["::1"]) | \
        set(EXPECTED["10.0.0.1"]) | set(EXPECTED['2001:db8::36']) | \
        set(EXPECTED["192.0.2.1"]) | set(EXPECTED['2001:db8::38'])
    assert all_names == expected


def test_nonlocal():
    ob = Hosts(context_wrap(HOSTS_EXAMPLE))
    expected = {
        "10.0.0.1": EXPECTED["10.0.0.1"],
        '2001:db8::36': EXPECTED['2001:db8::36'],
        '192.0.2.1': EXPECTED['192.0.2.1'],
        '2001:db8::38': EXPECTED['2001:db8::38'],
    }
    assert ob.get_nonlocal() == expected


def test_ip_of():
    ob = Hosts(context_wrap(HOSTS_EXAMPLE))
    # One IP with multiple names
    assert ob.ip_of('localhost4.localdomain4') == '127.0.0.1'
    # IPv6 address, with single name
    assert ob.ip_of('ipv6.example.com') == '2001:db8::36'
    # No host found = None
    assert ob.ip_of('foo.example.com') is None



from insights.parsers.sysconfig import DockerStorageSetupSysconfig
from insights.tests import context_wrap

DOCKER_STORAGE_SETUP1 = """
# Edit this file to override any configuration options specified in
# /usr/lib/docker-storage-setup/docker-storage-setup.
#
# For more details refer to "man docker-storage-setup"
VG=vgtest
AUTO_EXTEND_POOL=yes
##name = mydomain
POOL_AUTOEXTEND_THRESHOLD=60
POOL_AUTOEXTEND_PERCENT=20
""".strip()

DOCKER_STORAGE_SETUP2 = """
#comment
# comment
# comment = comment
# comment = comment = comment
#comment=comment
#comment=comment=comment
option_a=value_a
option_b = value_b
option_c= value_c
option_d =value_d
broken_option_e = value_e = value_2_e
broken_option_f=value_f=value_2_f
broken_option_g
option_h = value_h # some comment
option_i = value_i # this must be accessible, even after all these errors
""".strip()


def test_docker_storage_setup():
    context = context_wrap(DOCKER_STORAGE_SETUP1)
    result = DockerStorageSetupSysconfig(context)

    assert 'POOL_AUTOEXTEND_THRESHOLD' in result
    assert "20" == result["POOL_AUTOEXTEND_PERCENT"]
    assert "name" not in result
    assert "##name" not in result
    assert "vgtest" == result["VG"]

    context = context_wrap(DOCKER_STORAGE_SETUP2)
    result = DockerStorageSetupSysconfig(context)

    assert "comment" not in result
    assert "broken_option_g" not in result
    # Options with spaces around the '=' are not allowed in /etc/sysconfig
    assert "option_i" not in result



#!/usr/bin/python
import subprocess
import collections

ReturnTuple = collections.namedtuple('ReturnTuple',
                                     ['return_code', 'stdout', 'stderr'])


def subp(cmd):
    """
    Run a command as a subprocess.
    Return a triple of return code, standard out, standard err.
    """
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    out, err = proc.communicate()
    return ReturnTuple(proc.returncode, stdout=out, stderr=err)



"""
Combiner for `httpd -V` command
===============================

Combiner to get the valid parsed result of command `httpd -V`.

Examples:

    >>> HTTPDV2 = '''
    ... Server version: Apache/2.2.6 (Red Hat Enterprise Linux)
    ... Server's Module Magic Number: 20120211:24
    ... Server MPM:     Prefork
    ... Server compiled with....
    ... -D APR_HAS_SENDFILE
    ... -D APR_HAVE_IPV6 (IPv4-mapped addresses enabled)
    ... -D AP_TYPES_CONFIG_FILE="conf/mime.types"
    ... '''
    >>> HTTPDV1 = '''
    ... Server version: Apache/2.2.6 (Red Hat Enterprise Linux)
    ... Server's Module Magic Number: 20120211:24
    ... Server MPM:     Worker
    ... Server compiled with....
    ... -D APR_HAVE_IPV6 (IPv4-mapped addresses enabled)
    ... -D AP_TYPES_CONFIG_FILE="conf/mime.types"
    ... '''
    >>> PS_AUXWW = '''
    ... USER  PID %CPU %MEM    VSZ   RSS TTY STAT START   TIME COMMAND
    ... root   41  0.0  0.0  21452  1536 ?   Ss   Mar09   0:01 httpd.worker
    ... root   75  0.0  0.0      0     0 ?   S    Mar09   0:00 [kthreadd]
    ... '''
    >>> type(hv)
    <class 'insights.combiner.httpd_V.HttpdV'>
    >>> hv['Server MPM']
    'worker'
    >>> hv["Server's Module Magic Number"]
    '20120211:24'
    >>> 'APR_HAS_SENDFILE' in hv['Server compiled with']
    False
    >>> hv['Server compiled with']['APR_HAVE_IPV6']
    'IPv4-mapped addresses enabled'
"""

from insights.core.plugins import combiner
from insights.combiners.redhat_release import redhat_release
from insights.parsers.ps import PsAuxcww
from insights.parsers.httpd_V import HttpdV as HV
from insights.parsers.httpd_V import HttpdWorkerV as HWV
from insights.parsers.httpd_V import HttpdEventV as HEV
from insights import SkipComponent, LegacyItemAccess
from insights.util import deprecated


@combiner(requires=[redhat_release, PsAuxcww, [HV, HEV, HWV]])
class HttpdV(LegacyItemAccess):
    """
    .. warning::
        Deprecated combiner in the 3.0 framework, please use the
        :class:`insights.parsers.httpd_V.HttpdV` parser instead.

    A combiner to get the valid :class:`insights.parsers.httpd_V.HttpdV` on a
    rhel host.

    Attributes:
        data (dict): The bulk of the content is split on the colon and keys are
            kept as is.  Lines beginning with '-D' are kept in a dictionary
            keyed under 'Server compiled with'; each compilation option is a key
            in this sub-dictionary.  The value of the compilation options is the
            value after the equals sign, if one is present, or the value in
            brackets after the compilation option, or 'True' if only the
            compilation option is present.

    Raises:
        SkipComponent: When no valid HttpdV is found.
    """
    def __init__(self, rel, ps, hv, hev, hwv):
        deprecated(HttpdV, "Use the `HttpdV` parser in `insights.parsers.httpd_V`.")
        super(HttpdV, self).__init__()
        rhel_ver = rel.major
        self.data = hv.data if hv else None
        if rhel_ver == 6:
            if ps.fuzzy_match('httpd.worker'):
                self.data = hwv.data if hwv else None
            elif ps.fuzzy_match('httpd.event'):
                self.data = hev.data if hev else None
        if self.data is None:
            raise SkipComponent("Unable to get the valid `httpd -V` command")



"""
IscsiAdmModeSession - command ``iscsiadm - m session``
======================================================

This module provides the class ``IscsiAdmModeSession`` which processes
``iscsiadm -m session`` command output.  Typical output looks like::

    tcp: [1] 10.72.32.45:3260,1 iqn.2017-06.com.example:server1 (non-flash)
    tcp: [2] 10.72.32.45:3260,1 iqn.2017-06.com.example:server2 (non-flash)

The class has one attribute ``data`` which is a ``list`` representing each line
of the input data as a ``dict`` with keys corresponding to the keys in the output.

Examples:

    >>> iscsiadm_mode_session_content = '''
    ... tcp: [1] 10.72.32.45:3260,1 iqn.2017-06.com.example:server1 (non-flash)
    ... tcp: [2] 10.72.32.45:3260,1 iqn.2017-06.com.example:server2 (non-flash)
    ...'''.strip()

    >>> from insights.parsers.iscsiadm_mode_session import IscsiAdmModeSession
    >>> from insights.tests import context_wrap
    >>> shared = {IscsiAdmModeSession: IscsiAdmModeSession(context_wrap(iscsiadm_mode_session_content))}
    >>> result = shared[IscsiAdmModeSession]
    >>> result[0]
    {'IFACE_TRANSPORT': 'tcp', 'SID': '1', 'TARGET_IP': '10.72.32.45:3260,1',
     'TARGET_IQN': 'iqn.2017-06.com.example:server1'}
    >>> result[1]['TARGET_IQN']
    'iqn.2017-06.com.example:server2'
"""

from .. import parser, LegacyItemAccess, CommandParser
from insights.specs import Specs


@parser(Specs.iscsiadm_m_session)
class IscsiAdmModeSession(CommandParser, LegacyItemAccess):
    """Class to process the ``iscsiadm - m session`` command output.

    Attributes:

    data (list): A list containing a dictionary for each line of the output in the form::

            [
                {
                    'IFACE_TRANSPORT': "tcp",
                    'SID': '1',
                    'TARGET_IP': '10.72.32.45:3260,1',
                    'TARGET_IQN': 'iqn.2017-06.com.example:server1'
                },
                {
                    'IFACE_TRANSPORT': "tcp",
                    'SID': '2',
                    'TARGET_IP': '10.72.32.45:3260,1',
                    'TARGET_IQN': 'iqn.2017-06.com.example:server2'
                }
            ]
    """
    def parse_content(self, content):
        iscsiadm_session_all = []
        for line in (l.split() for l in content if l.strip()):
            iscsiadm_session_all.append({"IFACE_TRANSPORT": line[0].strip(':'),
                                         "SID": line[1].strip('[]'),
                                         "TARGET_IP": line[2],
                                         "TARGET_IQN": line[3]})

        self.data = iscsiadm_session_all
        """list of dict: List containing dict for each line of command output."""



from insights.parsers.secure import Secure
from insights.tests import context_wrap

from datetime import datetime

MSGINFO = """
Aug 24 09:31:39 localhost polkitd[822]: Loading rules from directory /etc/polkit-1/rules.d
Aug 24 09:31:39 localhost polkitd[822]: Loading rules from directory /usr/share/polkit-1/rules.d
Aug 24 09:31:39 localhost polkitd[822]: Finished loading, compiling and executing 6 rules
Aug 24 09:31:39 localhost polkitd[822]: Acquired the name org.freedesktop.PolicyKit1 on the system bus
Aug 25 13:52:54 localhost sshd[23085]: pam_unix(sshd:session): session opened for user zjj by (uid=0)
Aug 25 13:52:54 localhost sshd[23085]: error: openpty: No such file or directory
Aug 25 13:52:54 localhost sshd[23089]: error: session_pty_req: session 0 alloc failed
Aug 25 14:04:04 localhost sshd[23089]: Received disconnect from 10.66.192.100: 11: disconnected by user
Aug 25 14:04:04 localhost sshd[23085]: pam_unix(sshd:session): session closed for user zjj
Strange line with : but not otherwise parseable for testing code correctness
    Test continuation line
""".strip()


def test_secure():
    msg_info = Secure(context_wrap(MSGINFO))
    ssh_list = msg_info.get('sshd')
    assert 5 == len(ssh_list)
    assert ssh_list[0].get('timestamp') == "Aug 25 13:52:54"
    assert ssh_list[4].get('timestamp') == "Aug 25 14:04:04"
    polkitd = msg_info.get('Loading rules from directory')
    assert 2 == len(polkitd)
    assert polkitd[0].get('procname') == "polkitd[822]"
    assert polkitd[1].get(
        'raw_message') == "Aug 24 09:31:39 localhost polkitd[822]: Loading rules from directory /usr/share/polkit-1/rules.d"
    assert polkitd[1].get('message') == "Loading rules from directory /usr/share/polkit-1/rules.d"
    assert polkitd[1].get('hostname') == "localhost"

    # Test that non-parseable lines can still be found but just contain
    # the raw message and no other fields.
    assert msg_info.get('continuation') == [{
        'raw_message': '    Test continuation line'
    }]
    assert msg_info.get('Strange line') == [{
        'raw_message': 'Strange line with : but not otherwise parseable for testing code correctness',
        'message': 'but not otherwise parseable for testing code correctness'
    }]

    # Test get_after
    assert len(list(msg_info.get_after(datetime(2017, 8, 25, 0, 0, 0)))) == 7



from insights.parsers.postgresql_log import PostgreSQLLog
from insights.tests import context_wrap

POSTGRESQL_LOG = """
LOG:  unexpected EOF on client connection
LOG:  received fast shutdown request
LOG:  aborting any active transactions
FATAL:  terminating connection due to administrator command
FATAL:  terminating connection due to administrator command
LOG:  autovacuum launcher shutting down
FATAL:  terminating connection due to administrator command
LOG:  shutting down
LOG:  database system is shut down
LOG:  database system was shut down at 2015-11-10 18:36:41 EST
LOG:  database system is ready to accept connections
LOG:  autovacuum launcher started
"""


def test_postgresql_log():
    log = PostgreSQLLog(context_wrap(POSTGRESQL_LOG))
    assert "FATAL:  terminating connection" in log



from insights.parsers import krb5
from insights.tests import context_wrap


KRB5CONFIG = """
# Configuration snippets may be placed in this directory as well
includedir /etc/krb5.conf.d/
include /etc/krb5test.conf
module /etc/krb5test.conf:residual

[logging]
 default = FILE:/var/log/krb5libs.log
 kdc = FILE:/var/log/krb5kdc.log
 admin_server = FILE:/var/log/kadmind.log

[realms]
 dns_lookup_realm = false
 default_ccache_name = KEYRING:persistent:%{uid}
 default_ccache_name2 = KEYRING:%{uid}:persistent
 kdc_default_options = default.example.com
 kdc_default_options = default2.example.com
 EXAMPLE.COM = {
  kdc = kerberos.example.com
  admin_server = kerberos.example.com
  auth_to_local = RULE:[1:$1@$0](.*@.*EXAMPLE.ORG)s/@.*//
 }
 EXAMPLE4.COM = {
  kdc = kerberos.example4.com
  admin_server = kerberos.example4.com
 }
 ticket_lifetime = 24h
[libdefaults]
 dnsdsd = false
 tilnvs = 24h
 default_ccache_name = KEYRING:%{uid}:persistent
 EXAMPLE2.COM = {
  kdc = kerberos.example2.com
  admin_server = kerberos.example2.com
 }
 EXAMPLE3.COM = {
  kdc = kerberos.example3.com
  admin_server = kerberos.example3.com *
 }
# renew_lifetime = 7d
# forwardable = true
# rdns = false
""".strip()

KRB5CONFIG2 = """
# Configuration snippets may be placed in this directory as well
""".strip()

KRB5DCONFIG = """
# Configuration snippets may be placed in this directory as well

[logging]
 default = FILE:/var/log/krb5libs.log
 kdc = FILE:/var/log/krb5kdc.log

[realms]
 dns_lookup_realm = false
 ticket_lifetime = 24h
# default_ccache_name = KEYRING:persistent:%{uid}
 EXAMPLE.COM = {
  kdc = kerberos.example.com
  kdc = test2.example.com
  kdc = test3.example.com
  admin_server = kerberos.example.com
 }

[logging]
 default = FILE:/var/log/krb5libs.log
 kdc = FILE:/var/log/krb5kdc.log *
 admin_server = FILE:/var/log/kadmind.log

""".strip()

KRB5CONFIG3 = """
[logging]
 default = FILE:/var/log/krb5libs.log
 kdc = FILE:/var/log/krb5kdc.log
 admin_server = FILE:/var/log/kadmind.log

[libdefaults]
 dns_lookup_realm = false
 ticket_lifetime = 24h
 renew_lifetime = 7d
 forwardable = true
 rdns = false
# default_realm = EXAMPLE.COM
 default_ccache_name = KEYRING:persistent:%{uid}

[realms]
# EXAMPLE.COM = {
#  kdc = kerberos.example.com
#  admin_server = kerberos.example.com
# }

[domain_realm]
# .example.com = EXAMPLE.COM
# example.com = EXAMPLE.COM
""".strip()

KRB5_CONF_PATH = "etc/krb5.conf"
KRB5_DCONF_PATH = "etc/krb5.conf.d/test.conf"


def test_krb5configuration():
    common_conf_info = krb5.Krb5Configuration(context_wrap(KRB5CONFIG, path=KRB5_CONF_PATH))
    assert common_conf_info["libdefaults"]["dnsdsd"] == "false"
    assert "renew_lifetime" not in common_conf_info.data.keys()
    assert common_conf_info["realms"]["EXAMPLE.COM"]["kdc"] == "kerberos.example.com"
    assert common_conf_info["realms"]["default_ccache_name"] == "KEYRING:persistent:%{uid}"
    assert common_conf_info["libdefaults"]["default_ccache_name"] == "KEYRING:%{uid}:persistent"
    assert common_conf_info["realms"]["kdc_default_options"] == ["default.example.com", "default2.example.com"]
    assert "realms" in common_conf_info.sections()
    assert "realmstest" not in common_conf_info.sections()
    assert common_conf_info.has_section("realms")
    assert not common_conf_info.has_option("realms", "nosuchoption")
    assert not common_conf_info.has_option("nosucsection", "nosuchoption")
    assert not common_conf_info.options("realmsno")
    assert sorted(common_conf_info.options("logging")) == sorted(['default', 'admin_server', 'kdc'])
    assert common_conf_info.include == ["/etc/krb5test.conf"]
    assert common_conf_info.includedir == ["/etc/krb5.conf.d/"]
    assert common_conf_info.module == ["/etc/krb5test.conf:residual"]

    common_conf_info = krb5.Krb5Configuration(context_wrap(KRB5CONFIG3, path=KRB5_CONF_PATH))
    assert len(common_conf_info.sections()) == 4
    assert common_conf_info.has_section('domain_realm') is True
    assert sorted(common_conf_info.options('logging')) == sorted(['default', 'kdc', 'admin_server'])
    assert common_conf_info.has_option('libdefaults', 'dns_lookup_realm') is True
    assert common_conf_info.has_option('domain_realm', 'example.com') is False


def test2_krb5configuration():
    common_conf_info = krb5.Krb5Configuration(context_wrap(KRB5CONFIG2, path=KRB5_CONF_PATH))
    assert common_conf_info.data == {}


def test_krb5Dconfiguration():
    common_conf_info = krb5.Krb5Configuration(context_wrap(KRB5DCONFIG, path=KRB5_DCONF_PATH))
    assert common_conf_info["realms"]["ticket_lifetime"] == "24h"
    assert "default_ccache_name" not in common_conf_info.data.keys()
    assert common_conf_info["realms"]["EXAMPLE.COM"]["kdc"] == ['kerberos.example.com', 'test2.example.com', 'test3.example.com']
    assert not common_conf_info.has_option("logging", "admin_server")



from insights.parsers.sysconfig import DockerSysconfig
from insights.tests import context_wrap

DOCKER_SYSCONFIG_STD = """
# /etc/sysconfig/docker

# Modify these options if you want to change the way the docker daemon runs
OPTIONS='--selinux-enabled'

DOCKER_CERT_PATH=/etc/docker

# If you want to add your own registry to be used for docker search and docker
# pull use the ADD_REGISTRY option to list a set of registries, each prepended
# with --add-registry flag. The first registry added will be the first registry
# searched.
ADD_REGISTRY='--add-registry registry.access.redhat.com'

# If you want to block registries from being used, uncomment the BLOCK_REGISTRY
# option and give it a set of registries, each prepended with --block-registry
# flag. For example adding docker.io will stop users from downloading images
# from docker.io
# BLOCK_REGISTRY='--block-registry'

# If you have a registry secured with https but do not have proper certs
# distributed, you can tell docker to not look for full authorization by
# adding the registry to the INSECURE_REGISTRY line and uncommenting it.
# INSECURE_REGISTRY='--insecure-registry'

# On an SELinux system, if you remove the --selinux-enabled option, you
# also need to turn on the docker_transition_unconfined boolean.
# setsebool -P docker_transition_unconfined 1

# Location used for temporary files, such as those created by
# docker load and build operations. Default is /var/lib/docker/tmp
# Can be overriden by setting the following environment variable.
# DOCKER_TMPDIR=/var/tmp

# Controls the /etc/cron.daily/docker-logrotate cron job status.
# To disable, uncomment the line below.
# LOGROTATE=false
"""


def test_standard_content():
    context = context_wrap(DOCKER_SYSCONFIG_STD, 'etc/sysconfig/docker')
    sysconf = DockerSysconfig(context)

    assert sorted(sysconf.keys()) == sorted(['OPTIONS', 'DOCKER_CERT_PATH', 'ADD_REGISTRY'])
    assert 'OPTIONS' in sysconf
    assert sysconf['OPTIONS'] == '--selinux-enabled'
    assert sysconf.options == '--selinux-enabled'
    assert 'DOCKER_CERT_PATH' in sysconf
    assert sysconf['DOCKER_CERT_PATH'] == '/etc/docker'
    assert 'ADD_REGISTRY' in sysconf
    assert sysconf['ADD_REGISTRY'] == '--add-registry registry.access.redhat.com'
    assert sysconf.unparsed_lines == []



"""
Parsers for detection of Linux/Ebury 1.6 malware indicators
===========================================================

Libkeyutils - command ``find -L /lib /lib64 -name 'libkeyutils.so*'``
---------------------------------------------------------------------

Parses output of command ``find -L /lib /lib64 -name 'libkeyutils.so*'`` to find all potentially
affected libraries.

LibkeyutilsObjdumps - command ``find -L /lib /lib64 -name libkeyutils.so.1 -exec objdump -x "{}" \;``
-----------------------------------------------------------------------------------------------------

Parses output of command ``find -L /lib /lib64 -name libkeyutils.so.1 -exec objdump -x "{}" \;`` to
verify linked libraries.

https://www.welivesecurity.com/2017/10/30/windigo-ebury-update-2/
"""
import re

from .. import parser, CommandParser
from ..specs import Specs


@parser(Specs.libkeyutils)
class Libkeyutils(CommandParser):
    """
    This parser finds all 'libkeyutils.so*' libraries in either /lib or /lib64 directory and its
    sub-directories.

    Output of Command::

        /lib/libkeyutils.so.1
        /lib/tls/libkeyutils.so.1.6
        /lib64/libkeyutils.so

    Example::

        >>> shared[Libkeyutils].libraries
        ['/lib/libkeyutils.so.1', '/lib/tls/libkeyutils.so.1.6', '/lib64/libkeyutils.so']
    """
    def __init__(self, *args, **kwargs):
        self.libraries = []
        """list: all 'libkeyutils.so*' libraries located in either /lib or /lib64 directory and its sub-directories."""
        super(Libkeyutils, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        self.libraries = list(content)


@parser(Specs.libkeyutils_objdumps)
class LibkeyutilsObjdumps(CommandParser):
    """
    This parser goes through objdumps of all 'libkeyutils.so.1' libraries in either /lib or /lib64
    directory, and its sub-directories, to finds linked libraries.

    Output of Command::

        /lib/libkeyutils.so.1:     file format elf32-i386
        /lib/libkeyutils.so.1
        architecture: i386, flags 0x00000150:
        HAS_SYMS, DYNAMIC, D_PAGED
        start address 0x00000f80
        ...

        Dynamic Section:
          NEEDED               libdl.so.2
          NEEDED               libc.so.6
          NEEDED               libsbr.so
          SONAME               libkeyutils.so.1
          INIT                 0x00000e54
        ...


        /lib64/libkeyutils.so.1:     file format elf64-x86-64
        /lib64/libkeyutils.so.1
        architecture: i386:x86-64, flags 0x00000150:
        HAS_SYMS, DYNAMIC, D_PAGED
        start address 0x00000000000014b0
        ...

        Dynamic Section:
          NEEDED               libdl.so.2
          NEEDED               libsbr.so.6
          NEEDED               libfake.so
          SONAME               libkeyutils.so.1
          INIT                 0x0000000000001390
        ...

    Example::

        >>> shared[LibkeyutilsObjdumps].linked_libraries
        {'/lib/libkeyutils.so.1': ['libdl.so.2', 'libc.so.6', 'libsbr.so'],
         '/lib64/libkeyutils.so.1': ['libdl.so.2', 'libsbr.so.6', 'libfake.so']}
    """
    FILE_PATTERN = re.compile(r'(.*libkeyutils.so.1):\s*file format')
    NEED_PATTERN = re.compile(r'NEEDED\s+(.*)\s*$')

    def __init__(self, *args, **kwargs):
        self.linked_libraries = {}
        """dict: found libraries and their linked libraries."""
        super(LibkeyutilsObjdumps, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        file_name = None
        for line in content:
            r = LibkeyutilsObjdumps.FILE_PATTERN.search(line)
            if r:
                file_name = r.group(1)
            r = LibkeyutilsObjdumps.NEED_PATTERN.search(line)
            if r and file_name:
                library = r.group(1)
                if file_name not in self.linked_libraries:
                    self.linked_libraries[file_name] = [library]
                else:
                    self.linked_libraries[file_name].append(library)



from insights.tests import context_wrap
from insights.parsers.mlx4_port import Mlx4Port

MLX4_PORT = """
/sys/bus/pci/devices/0000:0c:00.0/mlx4_port1
ib
/sys/bus/pci/devices/0000:0c:00.0/mlx4_port2
eth
"""


def test_mlx4_port():
    result = Mlx4Port(context_wrap(MLX4_PORT))
    assert result.port_val == {'mlx4_port1': 'ib', 'mlx4_port2': 'eth'}
    assert result.port_val['mlx4_port1'] == 'ib'
    assert len(result.port_val) == 2



import pytest

from insights.core import JSONParser, ParseException
from insights.tests import context_wrap


class MyJsonParser(JSONParser):
    pass


def test_json_parser_success():
    ctx = context_wrap('{"a": "2"}')
    assert MyJsonParser(ctx)


def test_json_parser_failure():
    ctx = context_wrap("boom")
    with pytest.raises(ParseException) as ex:
        MyJsonParser(ctx)

    assert "MyJsonParser" in ex.value.args[0]



"""
ForemanTasksConfig - file ``/etc/sysconfig/foreman-tasks``
==========================================================
"""
from insights.specs import Specs
from insights.util import deprecated
from . import split_kv_pairs
from .. import LegacyItemAccess, Parser, get_active_lines, parser


@parser(Specs.foreman_tasks_config)
class ForemanTasksConfig(Parser, LegacyItemAccess):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.sysconfig.ForemanTasksSysconfig` instead.

    Parse the foreman-tasks configuration file.

    Produces a simple dictionary of keys and values from the configuration
    file contents , stored in the ``data`` attribute.  The object also
    functions as a dictionary itself thanks to the
    :py:class:`insights.core.LegacyItemAccess` mixin class.

    Sample configuration file::

        FOREMAN_USER=foreman
        BUNDLER_EXT_HOME=/usr/share/foreman
        RAILS_ENV=production
        FOREMAN_LOGGING=warn
        FOREMAN_LOGGING_SQL=warn
        FOREMAN_TASK_PARAMS="-p foreman"
        FOREMAN_LOG_DIR=/var/log/foreman

        RUBY_GC_MALLOC_LIMIT=4000100
        RUBY_GC_MALLOC_LIMIT_MAX=16000100
        RUBY_GC_MALLOC_LIMIT_GROWTH_FACTOR=1.1
        RUBY_GC_OLDMALLOC_LIMIT=16000100
        RUBY_GC_OLDMALLOC_LIMIT_MAX=16000100

        #Set the number of executors you want to run
        #EXECUTORS_COUNT=1

        #Set memory limit for executor process, before it's restarted automatically
        #EXECUTOR_MEMORY_LIMIT=2gb

        #Set delay before first memory polling to let executor initialize (in sec)
        #EXECUTOR_MEMORY_MONITOR_DELAY=7200 #default: 2 hours

        #Set memory polling interval, process memory will be checked every N seconds.
        #EXECUTOR_MEMORY_MONITOR_INTERVAL=60

    Examples:
        >>> foreman_tasks_config['RAILS_ENV']
        'production'
        >>> 'AUTO' in foreman_tasks_config
        False
    """
    def __init__(self, *args, **kwargs):
        deprecated(ForemanTasksConfig, "Import ForemanTasksSysconfig from insights.parsers.sysconfig instead")
        super(ForemanTasksConfig, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        self.data = split_kv_pairs(get_active_lines(content))



import doctest
import pytest
from insights.configtree import first, last
from insights.parsers import corosync, SkipException
from insights.tests import context_wrap

corosync_content = """
# Corosync init script configuration file

# COROSYNC_INIT_TIMEOUT specifies number of seconds to wait for corosync
# initialization (default is one minute).
COROSYNC_INIT_TIMEOUT=60

# COROSYNC_OPTIONS specifies options passed to corosync command
# (default is no options).
# See "man corosync" for detailed descriptions of the options.
COROSYNC_OPTIONS=""
"""

COROSYNC_CONF = """
totem {
    version: 2
    secauth: off
    cluster_name: tripleo_cluster
    transport: udpu
    token: 10000
}

nodelist {
    node {
        ring0_addr: overcloud-controller-0
        nodeid: 1
    }

    node {
        ring0_addr: overcloud-controller-1
        nodeid: 2
    }

    node {
        ring0_addr: overcloud-controller-2
        nodeid: 3
    }
}

quorum {
    provider: corosync_votequorum
}

logging {
    to_logfile: yes
    logfile: /var/log/cluster/corosync.log
    to_syslog: yes
}
""".strip()

COROSYNC_CONF_2 = """
totem {
    version: 2
    secauth: off
    cluster_name: tripleo_cluster
    transport: udpu
    token: 10000
}
""".strip()


def test_corosync_1():
    result = corosync.CoroSyncConfig(context_wrap(corosync_content))
    assert result.data['COROSYNC_OPTIONS'] == ""
    assert result.data['COROSYNC_INIT_TIMEOUT'] == "60"

    assert result.options == ''
    assert result.unparsed_lines == []


def test_corosync_conf():
    conf = corosync.CorosyncConf(context_wrap(COROSYNC_CONF))
    assert conf['totem']['token'][first].value == 10000
    assert conf['quorum']['provider'][first].value == 'corosync_votequorum'
    assert conf['nodelist']['node']['nodeid'][last].value == 3

    conf = corosync.CorosyncConf(context_wrap(COROSYNC_CONF_2))
    assert conf['totem']['version'][first].value == 2


def test_corosync_conf_empty():
    with pytest.raises(SkipException):
        assert corosync.CorosyncConf(context_wrap('')) is None


def test_doc_examples():
    failed, total = doctest.testmod(
        corosync,
        globs={'csconfig': corosync.CoroSyncConfig(context_wrap(corosync_content)),
               'corosync_conf': corosync.CorosyncConf(context_wrap(COROSYNC_CONF))}
    )
    assert failed == 0



from insights.tests import context_wrap
from insights.parsers import md5check
from insights.parsers import ParseException
import pytest

NORMAL_MD5_SAMPLE = """
d41d8cd98f00b204e9800998ecf8427e  /dev/null
7d4855248419b8a3ce6616bbc0e58301  /etc/localtime
910c11e6b93ae222b44143cb14f2b670  /usr/share/zoneinfo/America/Sao_Paulo
96db70199bdca045254f321a5e06d7b9  /usr/share/zoneinfo/America/Campo_Grande
ddc716ed2eac2ebfab07db289dd4270d  /usr/share/zoneinfo/America/Cuiaba
""".strip()

BAD_INPUT_SAMPLE = """
e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855  /dev/null
""".strip()

BAD_INPUT_MIX = """
d41d8cd98f00b204e9800998ecf8427e  /dev/null
/usr/bin/md5sum: /etc/pki/product/69.pem: No such file or directory
/usr/bin/md5sum: /etc/pki/product-default/69.pem: No such file or directory
""".strip()

BLANK_INPUT_SAMPLE = """
""".strip()

ONLY_NULL_SAMPLE = """
d41d8cd98f00b204e9800998ecf8427e  /dev/null
""".strip()

PRELINK_SAMPLE = """
b4949565c561187ebc69d21cf20b0207  /etc/pki/product/69.pem
""".strip()


def test_normal_md5():
    md5info = md5check.NormalMD5(context_wrap(NORMAL_MD5_SAMPLE))
    assert "/dev/null" not in md5info.files
    assert "/etc/localtime" in md5info.files
    assert "/usr/share/zoneinfo/America/Sao_Paulo" in md5info.files
    assert md5info['/etc/localtime'] == "7d4855248419b8a3ce6616bbc0e58301"
    assert md5info.get('/etc/hostname') is None
    assert "/etc/hostname" not in md5info.files


def test_normal_md5_blank_input():
    ctx = context_wrap(BLANK_INPUT_SAMPLE)
    with pytest.raises(ParseException) as sc:
        md5check.NormalMD5(ctx)
    assert "Input content is empty." in str(sc)


def test_normal_md5_bad_input():
    md5info = md5check.NormalMD5(context_wrap(BAD_INPUT_SAMPLE))
    assert md5info.data == {}
    assert md5info.files == []


def test_normal_md5_only_null():
    md5info = md5check.NormalMD5(context_wrap(ONLY_NULL_SAMPLE))
    assert md5info.data == {}
    assert md5info.files == []


def test_normal_md5_bad_mix():
    md5info = md5check.NormalMD5(context_wrap(BAD_INPUT_MIX))
    assert md5info.data == {}
    assert md5info.files == []


def test_prelink_md5():
    md5info = md5check.PrelinkMD5(context_wrap(PRELINK_SAMPLE))
    assert "/etc/pki/product/69.pem" in md5info.files
    assert md5info['/etc/pki/product/69.pem'] == "b4949565c561187ebc69d21cf20b0207"



"""
Mount - command ``/bin/mount``
==============================

This module provides parsing for the ``mount`` command. The ``Mount`` class
implements parsing for the ``mount`` command output which looks like::

    sysfs on /sys type sysfs (rw,nosuid,nodev,noexec,relatime,seclabel)
    proc on /proc type proc (rw,nosuid,nodev,noexec,relatime)
    /dev/mapper/HostVG-Config on /etc/shadow type ext4 (rw,noatime,seclabel,stripe=256,data=ordered)
    dev/sr0 on /run/media/root/VMware Tools type iso9660 (ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2) [VMware Tools]

The information is stored as a list of :class:`MountEntry` objects.  Each
:class:`MountEntry` object contains attributes for the following information that
are listed in the same order as in the command output:

 * ``filesystem`` - Name of filesystem
 * ``mount_point`` - Name of mount point for filesystem
 * ``mount_type`` - Name of filesystem type
 * ``mount_options`` - Mount options as a dictionary
 * ``mount_label`` - Only present if optional label is present
 * ``mount_clause`` - Full string from command output

The ``mount_options`` is wrapped as a :class:`MountOpts` object which contains
below fixed attributes:

* ``rw`` - Read write
* ``ro`` - Read only
* ``defaults`` - Use default options: rw, suid, dev, exec, auto, nouser, async, and relatime
* ``relatime`` - Inode access times relative to modify or change time
* ``seclabel`` - `seclabel` is enabled or not
* ``attr2`` - `opportunistic` improvement is enabled or not
* ``inode64`` - `inode64` is enabled or not
* ``noquota`` - Disk quotas are enforced or not

For instance, the option ``rw`` in ``(rw,dmode=0500)`` may be accessed as
``mnt_row_info.rw`` with the value ``True``, the ``dmode`` can be accessed as
``mnt_row_info.dmode`` with the value ``0500``.

MountEntry lines are also available in a ``mounts`` property, keyed on the
mount point.

Examples:
    >>> type(mnt_info)
    <class 'insights.parsers.mount.Mount'>
    >>> len(mnt_info)
    4
    >>> mnt_info[3].__dict__
    {'filesystem': 'dev/sr0',
     'mount_clause': 'dev/sr0 on /run/media/root/VMware Tools type iso9660 (ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2) [VMware Tools]',
     'mount_label': 'VMware Tools',
     'mount_options': {'dmode': '0500', 'relatime': True, 'uid': '0',
         'iocharset': 'utf8', 'gid': '0', 'mode': '0400', 'ro': True,
         'nosuid': True, 'uhelper': 'udisks2', 'nodev': True}
     'mount_point': '/run/media/root/VMware Tools',
     'mount_type': 'iso9660'}
    >>> mnt_info[3].filesystem
    'dev/sr0'
    >>> mnt_info[3].mount_type
    'iso9660'
    >>> mnt_info[3].mount_options
    {'dmode': '0500', 'gid': '0', 'iocharset': 'utf8', 'mode': '0400', 'nodev': True,
     'nosuid': True, 'relatime': True, 'ro': True, 'uhelper': 'udisks2', 'uid': '0'}
    >>> mnt_info[3].mount_options.ro
    True
    >>> mnt_info.mounts['/run/media/root/VMware Tools'].filesystem
    'dev/sr0'
"""

import re
from insights.specs import Specs
from ..parsers import optlist_to_dict, keyword_search, ParseException
from .. import parser, get_active_lines, LegacyItemAccess, CommandParser


class MountOpts(object):
    """
    An object representing the mount options found in mount or fstab entry.
    Each option in the comma-separated list is a key, and 'key=value'
    pairs such as 'gid=5' are split so that e.g. the key is 'gid' and the value
    is '5'.  Otherwise, the key is the option name and its value is 'True'.

    In addition, the following attributes will always be available and are
    False if not defined in the options list.

    Attributes:
        rw (bool): Read write
        ro (bool): Read only
        defaults: (bool): Use default options: rw, suid, dev, exec, auto, nouser, async, and relatime
        relatime (bool): Inode access times relative to modify or change time
        seclabel (bool): "seclabel" is enabled or not
        attr2 (bool): "opportunistic" improvement is enabled or not
        inode64 (bool): "inode64" is enabled or not
        noquota (bool): Disk quotas are enforced or not
    """
    attrs = {
        'rw': False,
        'ro': False,
        'defaults': False,
        'relatime': False,
        'seclabel': False,
        'attr2': False,
        'inode64': False,
        'noquota': False
    }

    def __init__(self, data={}):
        # Use '_data' but not 'data' since the 'data' could be an mount option
        self._data = data
        for k, v in MountOpts.attrs.items():
            if k not in data:
                setattr(self, k, v)
        for k, v in data.items():
            setattr(self, k, v)

    def __getitem__(self, item):
        return self._data[item]

    def __contains__(self, item):
        return item in self._data

    def get(self, item, default=None):
        """Returns value of key ``item`` in self._data or ``default``
        if key is not present.

        Parameters:
            item: Key to get from ``self.data``.
            default: Default value to return if key is not present.

        Returns:
            (str): String value of the stored item, or the default if not found.
        """
        return self._data.get(item, default)

    def items(self):
        """
        To keep backward compatibility and let it can be iterated as a
        dictionary.
        """
        for k, v in self._data.items():
            yield k, v


class MountEntry(LegacyItemAccess, CommandParser):
    """
    An object representing an entry in the output of ``mount`` command.  Each
    entry contains below fixed attributes:

    Attributes:
        mount_clause (str): Full string from command output
        filesystem (str): Name of filesystem
        mount_point (str): Name of mount point for filesystem
        mount_type (str): Name of filesystem type
        mount_options (MountOpts): Mount options as a :class:`insights.parsers.mount.MountOpts` object
    """
    attrs = {
            'mount_clause': '',
            'filesystem': '',
            'mount_point': '',
            'mount_type': '',
            'mount_options': MountOpts(),
    }

    def __init__(self, data={}):
        self.data = data
        for k, v in MountEntry.attrs.items():
            if k not in data:
                setattr(self, k, v)
        for k, v in data.items():
            setattr(self, k, v)

    def items(self):
        """
        To keep backward compatibility and let it can be iterated as a
        dictionary.
        """
        for k, v in self.data.items():
            yield k, v


@parser(Specs.mount)
class Mount(CommandParser):
    """Class of information for all output from ``mount`` command.

    Attributes:
        rows (list of MountEntry): List of :class:`MountEntry` objects for
            each row of the command output.

    Raises:
        ParseException: Raised when any problem parsing the command output.
    """

    def __len__(self):
        return len(self.rows)

    def __iter__(self):
        for row in self.rows:
            yield row

    def __getitem__(self, idx):
        if isinstance(idx, int):
            return self.rows[idx]
        elif isinstance(idx, str):
            return self.mounts[idx]
        else:
            raise TypeError("Mounts can only be indexed by mount string or line number")

    # /dev/mapper/fedora-home on /home type ext4 (rw,relatime,seclabel,data=ordered) [HOME]
    mount_line_re = r'^(?P<filesystem>\S+) on (?P<mount_point>.+?) type ' + \
        r'(?P<mount_type>\S+) \((?P<mount_options>[^)]+)\)' + \
        r'(?: \[(?P<mount_label>.*)\])?$'
    mount_line_rex = re.compile(mount_line_re)

    def parse_content(self, content):
        self.rows = []
        self.mounts = {}
        for line in get_active_lines(content):
            mount = {}
            mount['mount_clause'] = line
            match = self.mount_line_rex.search(line)
            if match:
                mount['filesystem'] = match.group('filesystem')
                mount['mount_point'] = match.group('mount_point')
                mount['mount_type'] = match.group('mount_type')
                mount_options = match.group('mount_options')
                mount['mount_options'] = MountOpts(optlist_to_dict(mount_options))
                if match.group('mount_label'):
                    mount['mount_label'] = match.group('mount_label')
            else:
                mount['parse_error'] = 'Unable to parse line'

            entry = MountEntry(mount)
            self.rows.append(entry)
            if match:
                self.mounts[mount['mount_point']] = entry

        if '/' not in self.mounts:
            raise ParseException("Input for mount must contain '/' mount point.")

    def get_dir(self, path):
        """
        MountEntry: returns the mount point that contains the given path.

        This finds the most specific mount path that contains the given path,
        by successively removing the directory or file name on the end of
        the path and seeing if that is a mount point.  This will always
        terminate since / is always a mount point.  Strings that are not
        absolute paths will return None.
        """
        import os
        while path != '':
            if path in self.mounts:
                return self.mounts[path]
            path = os.path.split(path)[0]
        return None

    def search(self, **kwargs):
        """
        Returns a list of the mounts (in order) matching the given criteria.
        Keys are searched for directly - see the
        :py:func:`insights.parsers.keyword_search` utility function for more
        details.  If no search parameters are given, no rows are returned.

        Examples:

            >>> mounts.search(filesystem='/dev/sda1')
            [{'filesystem': '/dev/sda1', 'mount_point': '/boot', ...}]
            >>> mounts.search(mount_options__contains='ro')
            [{'filesystem': '/dev/sr0', 'mount_point', '/mnt/CDROM', ...}, ...]

        Arguments:
            **kwargs (dict): Dictionary of key-value pairs to search for.

        Returns:
            (list): The list of mount points matching the given criteria.
        """
        return keyword_search(self.rows, **kwargs)



"""
Parsers for the Corosync Cluster Engine configurations
======================================================


Parsers included in this module are:

CoroSyncConfig - file ``/etc/sysconfig/corosync``
-------------------------------------------------

CorosyncConf - file ``/etc/corosync/corosync.conf``
---------------------------------------------------
"""
from insights.util import deprecated
from insights.core import ConfigParser
from insights.configtree.dictlike import DocParser, LineCounter
from insights.specs import Specs
from .. import SysconfigOptions, parser


@parser(Specs.corosync)
class CoroSyncConfig(SysconfigOptions):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.sysconfig.CorosyncSysconfig` instead.

    This parser reads the ``/etc/sysconfig/corosync`` file. It uses
    the ``SysconfigOptions`` parser class to convert the file into a
    dictionary of options. It also provides the ``options`` property
    as a helper to retrieve the ``COROSYNC_OPTIONS`` variable.

    Sample data::

        # Corosync init script configuration file

        # COROSYNC_INIT_TIMEOUT specifies number of seconds to wait for corosync
        # initialization (default is one minute).
        COROSYNC_INIT_TIMEOUT=60

        # COROSYNC_OPTIONS specifies options passed to corosync command
        # (default is no options).
        # See "man corosync" for detailed descriptions of the options.
        COROSYNC_OPTIONS=""

    Examples:

        >>> 'COROSYNC_OPTIONS' in csconfig.data
        True
        >>> csconfig.options
        ''
    """
    def __init__(self, *args, **kwargs):
        deprecated(CoroSyncConfig, "Import CorosyncSysconfig from insights.parsers.sysconfig instead")
        super(CoroSyncConfig, self).__init__(*args, **kwargs)

    @property
    def options(self):
        """ (str): The value of the ``COROSYNC_OPTIONS`` variable."""
        return self.data.get('COROSYNC_OPTIONS', '')


class CorosyncConfDocParser(DocParser):
    def parse_bare(self, pb):
        buf = []
        end = "{" + self.line_end
        while not pb.peek().isspace() and pb.peek() not in end:
            buf.append(next(pb).strip(':'))
        return "".join(buf)


def parse_doc(f, ctx=None, line_end="\n"):
    return CorosyncConfDocParser(ctx, line_end=line_end).parse_doc(LineCounter(f))


@parser(Specs.corosync_conf)
class CorosyncConf(ConfigParser):
    """Parse the output of the file ``/etc/corosync/corosync.conf`` using
    the ``ConfigParser`` base class. It exposes the corosync
    configuration through the configtree interface.

    The parameters in the directives are referred from the manpage of
    ``corosync.conf``. See ``man 8 corosync.conf`` for more info.

    Sample content of the file ``/etc/corosync/corosync.conf`` ::

        totem {
            version: 2
            secauth: off
            cluster_name: tripleo_cluster
            transport: udpu
            token: 10000
        }

        nodelist {
            node {
                ring0_addr: overcloud-controller-0
                nodeid: 1
            }

            node {
                ring0_addr: overcloud-controller-1
                nodeid: 2
            }

            node {
                ring0_addr: overcloud-controller-2
                nodeid: 3
            }
        }

        quorum {
            provider: corosync_votequorum
        }

        logging {
            to_logfile: yes
            logfile: /var/log/cluster/corosync.log
            to_syslog: yes
        }


    Example:

        >>> from insights.configtree import first, last
        >>> corosync_conf['quorum']['provider'][first].value
        'corosync_votequorum'
        >>> corosync_conf['totem']['token'][first].value
        10000
        >>> corosync_conf['nodelist']['node']['nodeid'][last].value
        3

    """
    def parse_doc(self, content):
        return parse_doc("\n".join(content), ctx=self)



#!/usr/bin/env python
"""
This module loads all of the components: specs, parsers and combiners, and
creates an `.rst` file output which can be included in the documentation
build.  The output file is a cross-reference for each type of component,
showing the component's dependents and dependencies.  In the case of
specs it only shows the dependents, and does not currently use the
context specific specs.

It can be called from the commandline to manually generate the file or
it may also be called inside the Sphinx *conf.py* `setup` function to
generate the documentation automatically.

Command Line Use
----------------

This assumes that you have a working installation of insights-core
and python environment for insights-core development.

To run from the command line simply execute this utility and provide
the name of the output file::

    $ python insights/util/component_graph.py output_filename.rst

"""

import argparse
from insights.core import dr
from insights.core.plugins import is_datasource, is_parser, is_combiner


def blank_line(fh, number=1):
    for l in range(number):
        fh.write("\n")


def print_info(info, fh):
    fh.write("* :py:obj:`{}`\n".format(info['name']))
    blank_line(fh)

    if info['dependents']:
        fh.write("  * Dependents:\n")
        blank_line(fh)
        for d in info['dependents']:
            fh.write("    * :py:obj:`{}`\n".format(d))
    else:
        fh.write("  * Dependents: None\n")
    blank_line(fh)

    if info['dependencies']:
        fh.write("  * Dependencies:\n")
        blank_line(fh)
        for d in info['dependencies']:
            fh.write("    * :py:obj:`{}`\n".format(d))
    else:
        fh.write("  * Dependencies: None\n")
    blank_line(fh)


def print_spec_info(info, fh):
    fh.write("* :py:obj:`{}`\n".format(info['name']))
    blank_line(fh)

    if info['dependents']:
        fh.write("  * Dependents:\n")
        blank_line(fh)
        for d in info['dependents']:
            fh.write("    * :py:obj:`{}`\n".format(d['name']))
            blank_line(fh)
            if d['dependents']:
                fh.write("      * Sub-Dependents:\n")
                blank_line(fh)
                for sd in d['dependents']:
                    fh.write("        * :py:obj:`{}`\n".format(sd))
            else:
                fh.write("      * Sub-Dependents: None\n")
            blank_line(fh)
    else:
        fh.write("  * Dependents: None\n")
    blank_line(fh)


def component_info(component):
    dependents = dr.get_dependents(component)
    dependencies = dr.get_dependencies(component)
    return {
        'name': dr.get_name(component),
        'dependents': [
            dr.get_name(c)
            for c in dependents
        ],
        'dependencies': [
            dr.get_name(c)
            for c in dependencies
        ]
    }


def main(filename):
    dr.load_components("insights.specs.default")
    dr.load_components("insights.specs.insights_archive")
    dr.load_components("insights.specs.sos_archive")
    dr.load_components("insights.specs.jdr_archive")
    dr.load_components("insights.parsers")
    dr.load_components("insights.combiners")

    parsers = sorted([c for c in dr.DELEGATES if is_parser(c)], key=dr.get_name)
    combiners = sorted([c for c in dr.DELEGATES if is_combiner(c)], key=dr.get_name)
    specs = sorted([c for c in dr.DELEGATES
                    if is_datasource(c) and dr.get_module_name(c) == 'insights.specs'],
                   key=dr.get_name)

    with open(filename, "w") as fh:
        fh.write("Components Cross-Reference\n")
        fh.write("==========================\n")

        fh.write("Specs Dependents\n")
        fh.write("----------------\n")
        for spec in specs:
            info = dict(name=dr.get_name(spec))
            info['dependents'] = []
            for d in dr.get_dependents(spec):
                info['dependents'].append({
                    'name': dr.get_name(d),
                    'dependents': [dr.get_name(sd) for sd in dr.get_dependents(d)]
                })
            print_spec_info(info, fh)

        blank_line(fh)
        fh.write("Parser Dependents/Dependencies\n")
        fh.write("------------------------------\n")
        for pars in parsers:
            print_info(component_info(pars), fh)

        blank_line(fh)
        fh.write("Combiner Dependents/Dependencies\n")
        fh.write("--------------------------------\n")
        for comb in combiners:
            print_info(component_info(comb), fh)


def parse_args():
    p = argparse.ArgumentParser()
    p.add_argument("out_filename", help="Name of the output file to write the cross-reference")
    return p.parse_args()


if __name__ == "__main__":
    args = parse_args()
    main(args.out_filename)



"""
MD5CheckSum - md5 checksums of specified binary or library files
================================================================

Module for processing output of the ``prelink -y --md5`` or ``md5sum`` command.

The name and md5 checksums of specified files are stored as lists. Each
entry will be processed by the shared parser.
"""

from .. import parser, CommandParser, LegacyItemAccess
from ..parsers import ParseException
from ..specs import Specs


class MD5CheckSum(LegacyItemAccess, CommandParser):
    """
    Common class to parse ``prelink -y --md5`` or ``md5sum`` command information.

    The output of these two commands contains two fields, the first is the
    md5 checksum and the second is the file name.

    Especially, we add /dev/null as one of the specified file to be checked,
    as ``md5sum`` command can read from stdin without feeding a file.

    Sample input for ``prelink -y --md5`` or ``md5sum`` command::

        d1e6613cfb62d3f111db7bdda39ac821  /usr/lib64/libsoftokn3.so

    Examples:

        >>> type(md5info)
        <class 'insights.parsers.md5check.MD5CheckSum'>
        >>> len(md5info.files)
        1
        >>> "/usr/lib64/libsoftokn3.so" in md5info.files
        True
        >>> md5info['/usr/lib64/libsoftokn3.so']
        'd1e6613cfb62d3f111db7bdda39ac821'

    Attributes:
        data (dict): All MD5 checksums are stored in this dictionary with the
                     filename as the key and the checksum as the value.
        files (list): Return the all file names in the checksum results.
    """

    def parse_content(self, content):
        if not content:
            raise ParseException("Input content is empty.")
        self.data = {}
        self.files = []
        for line in content:
            md5sum, filename = line.strip().split(' ', 1)
            md5sum, filename = md5sum.strip(), filename.strip()
            # skip the '/dev/null'
            if len(md5sum) == 32 and filename != "/dev/null":
                self.data[filename] = md5sum
        if self.data:
            self.files = self.data.keys()


@parser(Specs.prelink_orig_md5)
class PrelinkMD5(MD5CheckSum):
    """
    A parser for parsing the output of the ``prelink -y --md5``
    """
    pass


@parser(Specs.md5chk_files)
class NormalMD5(MD5CheckSum):
    """
    A parser for parsing the output of the ``md5sum``
    """
    pass



import pytest
from itertools import islice

from insights import tests
from insights.core.dr import get_name, load_components


def test_integration(component, compare_func, input_data, expected):
    actual = tests.run_test(component, input_data)
    compare_func(actual, expected)


def pytest_generate_tests(metafunc):
    pattern = pytest.config.getoption("-k")
    generate_tests(metafunc, test_integration, "insights/tests", pattern=pattern)


def generate_tests(metafunc, test_func, package_names, pattern=None):
    """
    This function hooks in to pytest's test collection framework and provides a
    test for every (input_data, expected) tuple that is generated from all
    @archive_provider-decorated functions.
    """
    if metafunc.function is test_func:
        if type(package_names) not in (list, tuple):
            package_names = [package_names]
        for package_name in package_names:
            load_components(package_name, include=pattern or ".*", exclude=None)
        args = []
        ids = []
        slow_mode = pytest.config.getoption("--runslow")
        fast_mode = pytest.config.getoption("--smokey")
        for f in tests.ARCHIVE_GENERATORS:
            ts = f(stride=1 if slow_mode else f.stride)
            if fast_mode:
                ts = islice(ts, 0, 1)
            for t in ts:
                args.append(t)
                input_data_name = t[2].name if not isinstance(t[2], list) else "multi-node"
                ids.append("#".join([get_name(f), input_data_name]))
        metafunc.parametrize("component,compare_func,input_data,expected", args, ids=ids)
